#include <stdio.h>
#include <math.h>
#include <mpi.h>
#include <stdlib.h>
#include <string.h>
#include <complex.h>

#include "physical_constants.h"
#include "materials.h"

#define M0 10000000
#define sigma_m 142000

int main(int argc, char** argv)
{

  FILE *system_info,*output_file;
  char dummy[255];
  char name_of_material[255];
  char source_type[255];
  char source_shape[255];
  char source_mode[4];
  int i,j,index,iteration;
  int node,p;
  int Number_of_materials;
  int MAXTIMESTEP,R_time_start,R_time_end;
  int nx,ny,px,py,npml,Bp=1;
  int source_xp,source_yp;
  int ob_x_1st,ob_y_1st,ob_x_2nd,ob_y_2nd;
  int c_of_pulse,w_of_pulse,w_of_beam;
  double cellsize,dT;
  double lambda,freq,omega,theta;


  MPI_Status status1,status2,status3,status4,status5,status6,status7,status8,status9,
	     status10,status11,status12,status13,status14,status15,status16;
  MPI_Request request1,request2,request3,request4,request5,request6,request7,request8,
	      request9,request10,request11,request12,request13,request14,request15,request16;
  MPI_Status sdsync1,sdsync2,sdsync3,sdsync4,sdsync5,sdsync6,sdsync7,sdsync8;
  MPI_Request rdsync1,rdsync2,rdsync3,rdsync4,rdsync5,rdsync6,rdsync7,rdsync8;

  /*
   * Configuration of System parameter    
   */

  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &node);
  MPI_Comm_size(MPI_COMM_WORLD, &p);
  p = p-1;

  system_info = fopen("system.info","r");

  if(node == 0) printf("\n# Part of configurations of materials\n");

  while(fscanf(system_info,"%s",dummy) != EOF)
  {
    if(strcmp(dummy,"Number_of_materials")==0)
    {
      fscanf(system_info,"%d",&Number_of_materials);
      //if(node == 0) printf(" - Number of materials : %d\n",Number_of_materials);
      break;
    }/* if(strcmp(dummy,"Number_of_materials")==0) */
  }/* while(fscanf(system_info,"%s",dummy) != EOF) */

  Materials * materials 
    = (Materials *)malloc(sizeof(Materials)*Number_of_materials);

  for(i=0;i<Number_of_materials;i++)
  {
    /* load permitivity */
    fscanf(system_info,"%s",dummy);
    sprintf(name_of_material,"%d.permitivity",i);
    if(strcmp(dummy,name_of_material)==0)
    {
      fscanf(system_info,"%lf",&materials[i].permitivity);
      if(node == 0) 
      {
	printf("\n  * materials[%d].permitivity = %f\n",i,materials[i].permitivity);
	materials[i].permitivity=materials[i].permitivity*EPSILON_0;
      }
      else
	materials[i].permitivity=materials[i].permitivity*EPSILON_0;
    }
    else
    {
      if(node == 0) printf("ERROR!! Loading materials[%d].permitivity!\n",i);
      exit(1);
    }

    /* load permiability */
    fscanf(system_info,"%s",dummy);
    sprintf(name_of_material,"%d.permiability",i);
    if(strcmp(dummy,name_of_material)==0)
    {
      fscanf(system_info,"%lf",&materials[i].permiability);
      if(node == 0) 
      {
	printf("  * materials[%d].permiability = %f\n",i,materials[i].permiability);
	materials[i].permiability=materials[i].permiability*MU_0;
      }
      else
	materials[i].permiability=materials[i].permiability*MU_0;
    }
    else
    {
      if(node == 0) printf("ERROR!! Loading materials[%d].permiability!\n",i);
      exit(1);
    }

    /* load relaxation rate for electric field */
    fscanf(system_info,"%s",dummy);
    sprintf(name_of_material,"%d.r_rate_elec",i);
    if(strcmp(dummy,name_of_material)==0)
    {
      fscanf(system_info,"%lf",&materials[i].r_rate_elec);
      if(node == 0) printf("  * materials[%d].r_rate_elec = %e\n",i,materials[i].r_rate_elec);
    }
    else
    {
      if(node == 0) printf("ERROR!! Loading materials[%d].r_rate_elec!\n",i);
      exit(1);
    }

    /* load plasma frequency for electric field */
    fscanf(system_info,"%s",dummy);
    sprintf(name_of_material,"%d.p_freq_elec",i);
    if(strcmp(dummy,name_of_material)==0)
    {
      fscanf(system_info,"%lf",&materials[i].p_freq_elec);
      if(node == 0) printf("  * materials[%d].p_freq_elec = %e\n",i,materials[i].p_freq_elec);
    }
    else
    {
      if(node == 0) printf("ERROR!! Loading materials[%d].p_freq_elec!\n",i);
      exit(1);
    }

    /* load relaxation rate for magnetic field */
    fscanf(system_info,"%s",dummy);
    sprintf(name_of_material,"%d.r_rate_mag",i);
    if(strcmp(dummy,name_of_material)==0)
    {
      fscanf(system_info,"%lf",&materials[i].r_rate_mag);
      if(node == 0) printf("  * materials[%d].r_rate_mag = %e\n",i,materials[i].r_rate_mag);
    }
    else
    {
      if(node == 0) printf("ERROR!! Loading materials[%d].r_rate_mag!\n",i);
      exit(1);
    }

    /* load plasma frequency for magnetic field */
    fscanf(system_info,"%s",dummy);
    sprintf(name_of_material,"%d.p_freq_mag",i);
    if(strcmp(dummy,name_of_material)==0)
    {
      fscanf(system_info,"%lf",&materials[i].p_freq_mag);
      if(node == 0) printf("  * materials[%d].p_freq_mag = %e\n",i,materials[i].p_freq_mag);
    }
    else
    {
      if(node == 0) printf("ERROR!! Loading materials[%d].p_freq_mag!\n",i);
      exit(1);
    }

  }/* for(i=0;i<Number_of_materials;i++) */

  if(node == 0) printf("\n# part of informations of geometry\n");

  while(fscanf(system_info,"%s",dummy) != EOF)
    if(strcmp(dummy,"Cells_in_X")==0) break;

  if(strcmp(dummy,"Cells_in_X")==0)
  {
    fscanf(system_info,"%d",&nx);
    if(node == 0) printf("  * Cells_in_X = %d\n",nx);
  }
  else
  {
    if(node == 0) printf("ERROR!! Loading Cells_in_X!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"Cells_in_Y")==0)
  {
    fscanf(system_info,"%d",&ny);
    if(node == 0) printf("  * Cells_in_Y = %d\n",ny);
  }
  else
  {
    if(node == 0) printf("ERROR!! Loading Cells_in_Y!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"Cells_in_PML")==0)
  {
    fscanf(system_info,"%d",&npml);
    if(node == 0) printf("  * Cells_in_PML = %d\n",npml);
    px=py=npml;
  }
  else
  {
    if(node == 0) printf("ERROR!! Loading Cells_in_PML!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"Size_of_cell")==0)
  {
    fscanf(system_info,"%lf",&cellsize);
    if(node == 0) printf("  * Size_of_cell = %lf\n",cellsize);
  }
  else
  {
    if(node == 0) printf("ERROR!! Loading Size_of_cell!\n");
    exit(1);
  }

  if(node == 0) printf("\n# Part of informations of input current source\n");

  while(fscanf(system_info,"%s",dummy) != EOF)
    if(strcmp(dummy,"Source_type")==0) break;

  if(strcmp(dummy,"Source_type")==0)
  {
    fscanf(system_info,"%s",source_type);
    if(node == 0) printf("  * Source_type = %s\n",source_type);
  }
  else
  {
    if(node == 0) printf("ERROR!! Loading Source_type!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"Source_shape")==0)
  {
    fscanf(system_info,"%s",source_shape);
    if(node == 0) printf("  * Source_shape = %s\n",source_shape);
  }
  else
  {
    if(node == 0) printf("ERROR!! Loading Source_shape!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"Source_mode")==0)
  {
    fscanf(system_info,"%s",source_mode);
    if(node == 0) printf("  * Source_mode= %s\n",source_shape);
  }
  else
  {
    if(node == 0) printf("ERROR!! Loading Source_mode!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"X-position")==0)
  {
    fscanf(system_info,"%d",&source_xp);
    if(node == 0) printf("  * X-position = %d\n",source_xp);
  }
  else
  {
    if(node == 0) printf("ERROR!! Loading X-position!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"Y-position")==0)
  {
    fscanf(system_info,"%d",&source_yp);
    if(node == 0) printf("  * Y-position = %d\n",source_yp);
  }
  else
  {
    if(node == 0) printf("ERROR!! Loading Y-position!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"Center_of_pulse")==0)
  {
    fscanf(system_info,"%d",&c_of_pulse);
    if(node == 0) printf("  * Center_of_pulse = %d\n",c_of_pulse);
  }
  else
  {
    if(node == 0) printf("ERROR!! Loading Center_of_pulse!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"Width_of_pulse")==0)
  {
    fscanf(system_info,"%d",&w_of_pulse);
    if(node == 0) printf("  * Width_of_pulse = %d\n",w_of_pulse);
  }
  else
  {
    if(node == 0) printf("ERROR!! Loading Width_of_pulse!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"Width_of_beam")==0)
  {
    fscanf(system_info,"%d",&w_of_beam);
    if(node == 0) printf("  * Width_of_beam = %d\n",w_of_beam);
  }
  else
  {
    if(node == 0) printf("ERROR!! Loading Width_of_beam!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"Omega")==0)
  {
    fscanf(system_info,"%lf",&omega);
    if(node == 0) printf("  * Omega = %e\n",omega);
  }
  else
  {
    if(node == 0) printf("ERROR!! Loading Omega!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"theta")==0)
  {
    fscanf(system_info,"%lf",&theta);
    if(node == 0) printf("  * Theta = %f\n",theta);
  }
  else
  {
    if(node == 0) printf("ERROR!! Loading Omega!\n");
    exit(1);
  }

  if(node == 0) printf("\n# Part of observing area\n");

  while(fscanf(system_info,"%s",dummy) != EOF)
    if(strcmp(dummy,"x_of_1st_point")==0) break;
  if(strcmp(dummy,"x_of_1st_point")==0)
  {
    fscanf(system_info,"%d",&ob_x_1st);
    if(node == 0) printf("  * X_of_1st_ob_point= %d\n",ob_x_1st);
  }
  else
  {
    if(node == 0) printf("ERROR!! X_of_1st_ob_point!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"y_of_1st_point")==0)
  {
    fscanf(system_info,"%d",&ob_y_1st);
    if(node == 0) printf("  * Y_of_1st_ob_point= %d\n",ob_y_1st);
  }
  else
  {
    if(node == 0) printf("ERROR!! Y_of_1st_ob_point!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"x_of_2nd_point")==0)
  {
    fscanf(system_info,"%d",&ob_x_2nd);
    if(node == 0) printf("  * X_of_2nd_ob_point= %d\n",ob_x_2nd);
  }
  else
  {
    if(node == 0) printf("ERROR!! X_of_2nd_ob_point!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"y_of_2nd_point")==0)
  {
    fscanf(system_info,"%d",&ob_y_2nd);
    if(node == 0) printf("  * Y_of_2nd_ob_point= %d\n",ob_y_2nd);
  }
  else
  {
    if(node == 0) printf("ERROR!! Y_of_2nd_ob_point!\n");
    exit(1);
  }

  if(node == 0) printf("\n# Part of calculation time\n");
  while(fscanf(system_info,"%s",dummy) != EOF)
    if(strcmp(dummy,"MAXTIMESTEP")==0) break;
  if(strcmp(dummy,"MAXTIMESTEP")==0)
  {
    fscanf(system_info,"%d",&MAXTIMESTEP);
    if(node == 0) printf("  * MAXTIMESTEP= %d\n",MAXTIMESTEP);
  }
  else
  {
    if(node == 0) printf("ERROR!! MAXTIMESTEP!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"R_time_start")==0)
  {
    fscanf(system_info,"%d",&R_time_start);
    if(node == 0) printf("  * R_time_start= %d\n",R_time_start);
  }
  else
  {
    if(node == 0) printf("ERROR!! R_time_start!\n");
    exit(1);
  }
  fscanf(system_info,"%s",dummy);
  if(strcmp(dummy,"R_time_end")==0)
  {
    fscanf(system_info,"%d",&R_time_end);
    if(node == 0) printf("  * R_time_end= %d\n",R_time_end);
  }
  else
  {
    if(node == 0) printf("ERROR!! R_time_end!\n");
    exit(1);
  }

  fclose(system_info);
  freq = omega / (2*M_PI);
  lambda = LIGHT_SPEED / freq;
  cellsize = cellsize * 1e-9;
  dT = cellsize/(sqrt(2.1)*LIGHT_SPEED);





  // Beam position select
  for(i=0;i<=p;i++)
  {
    if(Bp*nx/p > source_xp)
      break;
    else
      Bp++;
  }
  source_xp = source_xp - (Bp-1)*nx/p;

  ///////////////////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////////////////
  ////////////////////////////////     NODE 0 START    //////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////////////////
  if(node == 0)
  {
    // Total field_z
    double *total = (double *)malloc(sizeof(double)*nx*ny);
    double *N1 = (double *)malloc(sizeof(double)*(nx/p)*ny);
    double *N2 = (double *)malloc(sizeof(double)*(nx/p)*ny);
    double *N3 = (double *)malloc(sizeof(double)*(nx/p)*ny);
    double *N4 = (double *)malloc(sizeof(double)*(nx/p)*ny);
    double *N5 = (double *)malloc(sizeof(double)*(nx/p)*ny);
    double *N6 = (double *)malloc(sizeof(double)*(nx/p)*ny);
    double *N7 = (double *)malloc(sizeof(double)*(nx/p)*ny);
    double *N8 = (double *)malloc(sizeof(double)*(nx/p)*ny);
    double *N9 = (double *)malloc(sizeof(double)*(nx/p)*ny);
    double *N10 = (double *)malloc(sizeof(double)*(nx/p)*ny);
    double *N11 = (double *)malloc(sizeof(double)*(nx/p)*ny);
    double *N12 = (double *)malloc(sizeof(double)*(nx/p)*ny);
    /*
       double *N13 = (double *)malloc(sizeof(double)*(nx/p)*ny);
       double *N14 = (double *)malloc(sizeof(double)*(nx/p)*ny);
       double *N15 = (double *)malloc(sizeof(double)*(nx/p)*ny);
       double *N16 = (double *)malloc(sizeof(double)*(nx/p)*ny);
       */

    // sigma
    double *sigma_p = (double *)malloc(sizeof(double)*npml);

    // PML 1,2,3,4
    double *Cax_pc1 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Cay_pc1 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Cbx_pc1 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Cby_pc1 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Dax_pc1 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Day_pc1 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Dbx_pc1 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Dby_pc1 = (double *)malloc(sizeof(double)*(npml)*(npml));

    double *Cax_pc2 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Cay_pc2 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Cbx_pc2 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Cby_pc2 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Dax_pc2 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Day_pc2 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Dbx_pc2 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Dby_pc2 = (double *)malloc(sizeof(double)*(npml)*(npml));

    double *Cax_pc3 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Cay_pc3 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Cbx_pc3 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Cby_pc3 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Dax_pc3 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Day_pc3 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Dbx_pc3 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Dby_pc3 = (double *)malloc(sizeof(double)*(npml)*(npml));

    double *Cax_pc4 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Cay_pc4 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Cbx_pc4 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Cby_pc4 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Dax_pc4 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Day_pc4 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Dbx_pc4 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Dby_pc4 = (double *)malloc(sizeof(double)*(npml)*(npml));

    // PML 6,8
    double *Ca_ps6 = (double *)malloc(sizeof(double)*(npml)*(ny));
    double *Cb_ps6 = (double *)malloc(sizeof(double)*(npml)*(ny));
    double *Da_ps6 = (double *)malloc(sizeof(double)*(npml)*(ny));
    double *Db_ps6 = (double *)malloc(sizeof(double)*(npml)*(ny));
    double *Ca_ps8 = (double *)malloc(sizeof(double)*(npml)*(ny));
    double *Cb_ps8 = (double *)malloc(sizeof(double)*(npml)*(ny));
    double *Da_ps8 = (double *)malloc(sizeof(double)*(npml)*(ny));
    double *Db_ps8 = (double *)malloc(sizeof(double)*(npml)*(ny));

    // Fields

    // PML corner
    double *Ex_pc1 = (double *)malloc(sizeof(double)*(npml)*(npml+1));
    double *Ey_pc1 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Hzx_pc1 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Hzy_pc1 = (double *)malloc(sizeof(double)*(npml)*(npml));

    double *Ex_pc2 = (double *)malloc(sizeof(double)*(npml)*(npml+1));
    double *Ey_pc2 = (double *)malloc(sizeof(double)*(npml+1)*(npml));
    double *Hzx_pc2 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Hzy_pc2 = (double *)malloc(sizeof(double)*(npml)*(npml));

    double *Ex_pc3 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Ey_pc3 = (double *)malloc(sizeof(double)*(npml+1)*(npml));
    double *Hzx_pc3 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Hzy_pc3 = (double *)malloc(sizeof(double)*(npml)*(npml));

    double *Ex_pc4 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Ey_pc4 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Hzx_pc4 = (double *)malloc(sizeof(double)*(npml)*(npml));
    double *Hzy_pc4 = (double *)malloc(sizeof(double)*(npml)*(npml));

    // PML 6,8
    double *Ex_ps6 = (double *)malloc(sizeof(double)*(npml)*(ny));
    double *Ey_ps6 = (double *)malloc(sizeof(double)*(npml+1)*(ny));
    double *Hzx_ps6 = (double *)malloc(sizeof(double)*(npml)*(ny));
    double *Hzy_ps6 = (double *)malloc(sizeof(double)*(npml)*(ny));

    double *Ex_ps8 = (double *)malloc(sizeof(double)*(npml)*(ny));
    double *Ey_ps8 = (double *)malloc(sizeof(double)*(npml)*(ny));
    double *Hzx_ps8 = (double *)malloc(sizeof(double)*(npml)*(ny));
    double *Hzy_ps8 = (double *)malloc(sizeof(double)*(npml)*(ny));

    // shared components
    // send
    double *Hzx_sndu = (double *)malloc(sizeof(double)*npml);
    double *Hzy_sndu = (double *)malloc(sizeof(double)*npml);
    double *Hz_snd = (double *)malloc(sizeof(double)*ny);
    double *Hzx_sndb = (double *)malloc(sizeof(double)*npml);
    double *Hzy_sndb = (double *)malloc(sizeof(double)*npml);
    double *Ey_sndu = (double *)malloc(sizeof(double)*npml);
    double *Ey_snd = (double *)malloc(sizeof(double)*ny);
    double *Ey_sndb = (double *)malloc(sizeof(double)*npml);
    // receive
    double *Ey_rcvu = (double *)malloc(sizeof(double)*npml);
    double *Ey_rcv = (double *)malloc(sizeof(double)*ny);
    double *Ey_rcvb = (double *)malloc(sizeof(double)*npml);
    double *Hzx_rcvu = (double *)malloc(sizeof(double)*npml);
    double *Hzy_rcvu = (double *)malloc(sizeof(double)*npml);
    double *Hz_rcv = (double *)malloc(sizeof(double)*ny);
    double *Hzx_rcvb = (double *)malloc(sizeof(double)*npml);
    double *Hzy_rcvb = (double *)malloc(sizeof(double)*npml);

    char pic_file[256];

    printf("=================================\n");
    printf("Program Start\n");
    printf("=================================\n");
    printf("System information : \n");
    printf("\t- Mode : %s\n",source_mode);
    printf("\t- Max time-step : %d\n",MAXTIMESTEP);
    printf("\t- Calculation space width(number of cells) : %d\n",nx);
    printf("\t- Calculation space height(number of cells) : %d\n",ny);
    printf("\t- PML depth(number of cells) : %d\n",npml);
    printf("\t- Input beam frequency : %e\n",freq);
    printf("\t- Input beam lambda : %e\n",lambda);
    printf("\t- CELL SIZE : %e\n",cellsize);
    printf("\t- Delta t : %e\n",dT);
    printf("\t- Node length : %d\n",nx/p);
    printf("\t- source_xp : %d\n",source_xp);
    printf("\t- source_yp : %d\n",source_yp);
    printf("\t- w_of_beam : %d\n",w_of_beam);
    printf("\t- Bp : %d\n",Bp);
    printf("=================================\n");

    for(i=0;i<nx;i++)
      for(j=0;j<ny;j++)
	total[i*ny+j] = 0.0;
    for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
      {
	N1[i*ny+j] = 0.0;
	N2[i*ny+j] = 0.0;
	N3[i*ny+j] = 0.0;
	N4[i*ny+j] = 0.0;
	N5[i*ny+j] = 0.0;
	N6[i*ny+j] = 0.0;
	N7[i*ny+j] = 0.0;
	N8[i*ny+j] = 0.0;
	N9[i*ny+j] = 0.0;
	N10[i*ny+j] = 0.0;
	N11[i*ny+j] = 0.0;
	N12[i*ny+j] = 0.0;
	/*
	   N13[i*ny+j] = 0.0;
	   N14[i*ny+j] = 0.0;
	   N15[i*ny+j] = 0.0;
	   N16[i*ny+j] = 0.0;
	   */
      }

    for(i=0;i<ny;i++)
    {
      if(i<npml)
      {
	Hzx_sndu[i] = 0.0;
	Hzy_sndu[i] = 0.0;
	Hz_snd[i] = 0.0;
	Hzx_sndb[i] = 0.0;
	Hzy_sndb[i] = 0.0;
	Ey_sndu[i] = 0.0;
	Ey_snd[i] = 0.0;
	Ey_sndb[i] = 0.0;
	Ey_rcvu[i] = 0.0;
	Ey_rcv[i] = 0.0;
	Ey_rcvb[i] = 0.0;
	Hzx_rcvu[i] = 0.0;
	Hzy_rcvu[i] = 0.0;
	Hz_rcv[i] = 0.0;
	Hzx_rcvb[i] = 0.0;
	Hzy_rcvb[i] = 0.0;
      }
      else
      {
	Hz_snd[i] = 0.0;
	Hz_rcv[i] = 0.0;
      }
    }


    for(i=0;i<npml;i++) 
      sigma_p[i] = (double)((double)sigma_m*(double)((i+1)*(i+1)*(i+1)))/(double)(npml*npml*npml);
    sigma_p[0] = ((sigma_p[0] - 0)/2.0 + sigma_p[0] + (sigma_p[0] + sigma_p[1])/2.0)/3.0;
    for(i=1;i<npml-1;i++) 
      sigma_p[i] = ((sigma_p[i-1]+sigma_p[i])/2.0 + sigma_p[i] + (sigma_p[i] + sigma_p[i+1])/2.0)/3.0;

    for(i=0;i<npml+1;i++)
      for(j=0;j<npml+1;j++)
      {
	if(i==npml)
	{
	  Ey_pc2[i*npml+j] = 0.0;
	  Ey_pc3[i*npml+j] = 0.0;
	}
	else if(j==npml)
	{
	  Ex_pc1[i*(npml+1)+j] = 0.0;
	  Ex_pc2[i*(npml+1)+j] = 0.0;
	}
	else
	{
	  Ex_pc1[i*(npml+1)+j] = 0.0; 
	  Ey_pc1[i*npml+j] = 0.0; 
	  Hzx_pc1[i*npml+j] = 0.0; 
	  Hzy_pc1[i*npml+j] = 0.0;
	  Ex_pc2[i*(npml+1)+j] = 0.0;
	  Ey_pc2[i*npml+j] = 0.0;
	  Hzx_pc2[i*npml+j] = 0.0;
	  Hzy_pc2[i*npml+j] = 0.0;
	  Ex_pc3[i*npml+j] = 0.0;
	  Ey_pc3[i*npml+j] = 0.0;
	  Hzx_pc3[i*npml+j] = 0.0;
	  Hzy_pc3[i*npml+j] = 0.0;
	  Ex_pc4[i*npml+j] = 0.0;
	  Ey_pc4[i*npml+j] = 0.0;
	  Hzx_pc4[i*npml+j] = 0.0;
	  Hzy_pc4[i*npml+j] = 0.0;
	}
      }
    for(i=0;i<npml+1;i++)
      for(j=0;j<ny;j++)
      {
	if(i==npml)
	  Ey_ps6[i*ny+j] = 0.0;
	else
	{
	  Ex_ps6[i*ny+j] = 0.0;
	  Ey_ps6[i*ny+j] = 0.0;
	  Hzx_ps6[i*ny+j] = 0.0;
	  Hzy_ps6[i*ny+j] = 0.0;
	  Ex_ps8[i*ny+j] = 0.0;
	  Ey_ps8[i*ny+j] = 0.0;
	  Hzx_ps8[i*ny+j] = 0.0;
	  Hzy_ps8[i*ny+j] = 0.0;
	}
      }


    if(strcmp(source_mode,"TE")==0)
    {
      // PML 1,2,3,4
      for(i=0;i<npml;i++)
	for(j=0;j<npml;j++)
	{
	  Cax_pc1[i*npml+j] = (1-sigma_p[npml-1-i]*dT/(2*EPSILON_0))/(1+sigma_p[npml-1-i]*dT/(2*EPSILON_0));
	  Cay_pc1[i*npml+j] = (1-sigma_p[j]*dT/(2*EPSILON_0))/(1+sigma_p[j]*dT/(2*EPSILON_0));
	  Cbx_pc1[i*npml+j] = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[npml-1-i]*dT/(2*EPSILON_0));
	  Cby_pc1[i*npml+j] = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[j]*dT/(2*EPSILON_0));
	  Dax_pc1[i*npml+j] = (1-SH*sigma_p[npml-1-i]*dT/(2*MU_0))/(1+SH*sigma_p[npml-1-i]*dT/(2*MU_0));
	  Day_pc1[i*npml+j] = (1-SH*sigma_p[j]*dT/(2*MU_0))/(1+SH*sigma_p[j]*dT/(2*MU_0));
	  Dbx_pc1[i*npml+j] = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[npml-1-i]*dT/(2*MU_0));
	  Dby_pc1[i*npml+j] = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[j]*dT/(2*MU_0));

	  Cax_pc2[i*npml+j] = (1-sigma_p[i]*dT/(2*EPSILON_0))/(1+sigma_p[i]*dT/(2*EPSILON_0));
	  Cay_pc2[i*npml+j] = (1-sigma_p[j]*dT/(2*EPSILON_0))/(1+sigma_p[j]*dT/(2*EPSILON_0));
	  Cbx_pc2[i*npml+j] = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[i]*dT/(2*EPSILON_0));
	  Cby_pc2[i*npml+j] = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[j]*dT/(2*EPSILON_0));
	  Dax_pc2[i*npml+j] = (1-SH*sigma_p[i]*dT/(2*MU_0))/(1+SH*sigma_p[i]*dT/(2*MU_0));
	  Day_pc2[i*npml+j] = (1-SH*sigma_p[j]*dT/(2*MU_0))/(1+SH*sigma_p[j]*dT/(2*MU_0));
	  Dbx_pc2[i*npml+j] = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[i]*dT/(2*MU_0));
	  Dby_pc2[i*npml+j] = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[j]*dT/(2*MU_0));

	  Cax_pc3[i*npml+j] = (1-sigma_p[i]*dT/(2*EPSILON_0))/(1+sigma_p[i]*dT/(2*EPSILON_0));
	  Cay_pc3[i*npml+j] = (1-sigma_p[npml-1-j]*dT/(2*EPSILON_0))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0));
	  Cbx_pc3[i*npml+j] = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[i]*dT/(2*EPSILON_0));
	  Cby_pc3[i*npml+j] = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0));
	  Dax_pc3[i*npml+j] = (1-SH*sigma_p[i]*dT/(2*MU_0))/(1+SH*sigma_p[i]*dT/(2*MU_0));
	  Day_pc3[i*npml+j] = (1-SH*sigma_p[npml-1-j]*dT/(2*MU_0))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0));
	  Dbx_pc3[i*npml+j] = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[i]*dT/(2*MU_0));
	  Dby_pc3[i*npml+j] = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0));

	  Cax_pc4[i*npml+j] = (1-sigma_p[npml-1-i]*dT/(2*EPSILON_0))/(1+sigma_p[npml-1-i]*dT/(2*EPSILON_0));
	  Cay_pc4[i*npml+j] = (1-sigma_p[npml-1-j]*dT/(2*EPSILON_0))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0));
	  Cbx_pc4[i*npml+j] = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[npml-1-i]*dT/(2*EPSILON_0));
	  Cby_pc4[i*npml+j] = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0));
	  Dax_pc4[i*npml+j] = (1-SH*sigma_p[npml-1-i]*dT/(2*MU_0))/(1+SH*sigma_p[npml-1-i]*dT/(2*MU_0));
	  Day_pc4[i*npml+j] = (1-SH*sigma_p[npml-1-j]*dT/(2*MU_0))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0));
	  Dbx_pc4[i*npml+j] = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[npml-1-i]*dT/(2*MU_0));
	  Dby_pc4[i*npml+j] = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0));
	}

      // PML 6,8 
      for(i=0;i<npml;i++) 
	for(j=0;j<ny;j++) 
	{ 
	  Ca_ps6[i*ny+j] = (1-sigma_p[i]*dT/(2*EPSILON_0))/(1+sigma_p[i]*dT/(2*EPSILON_0)); 
	  Cb_ps6[i*ny+j] = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[i]*dT/(2*EPSILON_0)); 
	  Da_ps6[i*ny+j] = (1-SH*sigma_p[i]*dT/(2*MU_0))/(1+SH*sigma_p[i]*dT/(2*MU_0)); 
	  Db_ps6[i*ny+j] = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[i]*dT/(2*MU_0)); 

	  Ca_ps8[i*ny+j] = (1-sigma_p[npml-1-i]*dT/(2*EPSILON_0))/(1+sigma_p[npml-1-i]*dT/(2*EPSILON_0)); 
	  Cb_ps8[i*ny+j] = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[npml-1-i]*dT/(2*EPSILON_0)); 
	  Da_ps8[i*ny+j] = (1-SH*sigma_p[npml-1-i]*dT/(2*MU_0))/(1+SH*sigma_p[npml-1-i]*dT/(2*MU_0)); 
	  Db_ps8[i*ny+j] = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[npml-1-i]*dT/(2*MU_0)); 
	}
    }// end of if(strcmp(source_mode,"TE")==0)

    if(strcmp(source_mode,"TM")==0)
    {
      // PML 1,2,3,4
      for(i=0;i<npml;i++)
	for(j=0;j<npml;j++)
	{
	  Dax_pc1[i*npml+j] = (1-sigma_p[npml-1-i]*dT/(2*EPSILON_0))/(1+sigma_p[npml-1-i]*dT/(2*EPSILON_0));
	  Day_pc1[i*npml+j] = (1-sigma_p[j]*dT/(2*EPSILON_0))/(1+sigma_p[j]*dT/(2*EPSILON_0));
	  Dbx_pc1[i*npml+j] = -sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[npml-1-i]*dT/(2*EPSILON_0));
	  Dby_pc1[i*npml+j] = -sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[j]*dT/(2*EPSILON_0));
	  Cax_pc1[i*npml+j] = (1-SH*sigma_p[npml-1-i]*dT/(2*MU_0))/(1+SH*sigma_p[npml-1-i]*dT/(2*MU_0));
	  Cay_pc1[i*npml+j] = (1-SH*sigma_p[j]*dT/(2*MU_0))/(1+SH*sigma_p[j]*dT/(2*MU_0));
	  Cbx_pc1[i*npml+j] = -sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[npml-1-i]*dT/(2*MU_0));
	  Cby_pc1[i*npml+j] = -sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[j]*dT/(2*MU_0));

	  Dax_pc2[i*npml+j] = (1-sigma_p[i]*dT/(2*EPSILON_0))/(1+sigma_p[i]*dT/(2*EPSILON_0));
	  Day_pc2[i*npml+j] = (1-sigma_p[j]*dT/(2*EPSILON_0))/(1+sigma_p[j]*dT/(2*EPSILON_0));
	  Dbx_pc2[i*npml+j] = -sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[i]*dT/(2*EPSILON_0));
	  Dby_pc2[i*npml+j] = -sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[j]*dT/(2*EPSILON_0));
	  Cax_pc2[i*npml+j] = (1-SH*sigma_p[i]*dT/(2*MU_0))/(1+SH*sigma_p[i]*dT/(2*MU_0));
	  Cay_pc2[i*npml+j] = (1-SH*sigma_p[j]*dT/(2*MU_0))/(1+SH*sigma_p[j]*dT/(2*MU_0));
	  Cbx_pc2[i*npml+j] = -sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[i]*dT/(2*MU_0));
	  Cby_pc2[i*npml+j] = -sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[j]*dT/(2*MU_0));

	  Dax_pc3[i*npml+j] = (1-sigma_p[i]*dT/(2*EPSILON_0))/(1+sigma_p[i]*dT/(2*EPSILON_0));
	  Day_pc3[i*npml+j] = (1-sigma_p[npml-1-j]*dT/(2*EPSILON_0))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0));
	  Dbx_pc3[i*npml+j] = -sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[i]*dT/(2*EPSILON_0));
	  Dby_pc3[i*npml+j] = -sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0));
	  Cax_pc3[i*npml+j] = (1-SH*sigma_p[i]*dT/(2*MU_0))/(1+SH*sigma_p[i]*dT/(2*MU_0));
	  Cay_pc3[i*npml+j] = (1-SH*sigma_p[npml-1-j]*dT/(2*MU_0))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0));
	  Cbx_pc3[i*npml+j] = -sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[i]*dT/(2*MU_0));
	  Cby_pc3[i*npml+j] = -sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0));

	  Dax_pc4[i*npml+j] = (1-sigma_p[npml-1-i]*dT/(2*EPSILON_0))/(1+sigma_p[npml-1-i]*dT/(2*EPSILON_0));
	  Day_pc4[i*npml+j] = (1-sigma_p[npml-1-j]*dT/(2*EPSILON_0))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0));
	  Dbx_pc4[i*npml+j] = -sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[npml-1-i]*dT/(2*EPSILON_0));
	  Dby_pc4[i*npml+j] = -sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0));
	  Cax_pc4[i*npml+j] = (1-SH*sigma_p[npml-1-i]*dT/(2*MU_0))/(1+SH*sigma_p[npml-1-i]*dT/(2*MU_0));
	  Cay_pc4[i*npml+j] = (1-SH*sigma_p[npml-1-j]*dT/(2*MU_0))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0));
	  Cbx_pc4[i*npml+j] = -sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[npml-1-i]*dT/(2*MU_0));
	  Cby_pc4[i*npml+j] = -sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0));
	}

      // PML 6,8 
      for(i=0;i<npml;i++) 
	for(j=0;j<ny;j++) 
	{ 
	  Da_ps6[i*ny+j] = (1-sigma_p[i]*dT/(2*EPSILON_0))/(1+sigma_p[i]*dT/(2*EPSILON_0)); 
	  Db_ps6[i*ny+j] = -sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[i]*dT/(2*EPSILON_0)); 
	  Ca_ps6[i*ny+j] = (1-SH*sigma_p[i]*dT/(2*MU_0))/(1+SH*sigma_p[i]*dT/(2*MU_0)); 
	  Cb_ps6[i*ny+j] = -sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[i]*dT/(2*MU_0)); 

	  Da_ps8[i*ny+j] = (1-sigma_p[npml-1-i]*dT/(2*EPSILON_0))/(1+sigma_p[npml-1-i]*dT/(2*EPSILON_0)); 
	  Db_ps8[i*ny+j] = -sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[npml-1-i]*dT/(2*EPSILON_0)); 
	  Ca_ps8[i*ny+j] = (1-SH*sigma_p[npml-1-i]*dT/(2*MU_0))/(1+SH*sigma_p[npml-1-i]*dT/(2*MU_0)); 
	  Cb_ps8[i*ny+j] = -sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[npml-1-i]*dT/(2*MU_0)); 
	}
    }// end of if(strcmp(source_mode,"TM")==0)

    for(iteration=0;iteration<MAXTIMESTEP;iteration++)
    {
      if(iteration % 7 == 0)
      {
	printf("\riterations = %3.0f%%  complete    - ",100*(double)iteration/MAXTIMESTEP);
	fflush(stdout);
      }
      if(iteration % 7 == 2)
      {
	printf("\riterations = %3.0f%%  complete    \\ ",100*(double)iteration/MAXTIMESTEP);
	fflush(stdout);
      }
      if(iteration % 7 == 4)
      {
	printf("\riterations = %3.0f%%  complete    / ",100*(double)iteration/MAXTIMESTEP);
	fflush(stdout);
      }
      // data sync.
      for(i=0;i<npml;i++)
      {
	Ey_sndu[i] = Ey_pc2[i];
	Ey_sndb[i] = Ey_pc3[i];
      }
      for(i=0;i<ny;i++)
	Ey_snd[i] = Ey_ps6[i];

      MPI_Irecv(Ey_rcvu,npml,MPI_DOUBLE,1,101,MPI_COMM_WORLD,&rdsync1);
      MPI_Irecv(Ey_rcv,ny,MPI_DOUBLE,1,102,MPI_COMM_WORLD,&rdsync2);
      MPI_Irecv(Ey_rcvb,npml,MPI_DOUBLE,1,103,MPI_COMM_WORLD,&rdsync3);

      MPI_Send(Ey_sndu,npml,MPI_DOUBLE,p,1,MPI_COMM_WORLD);
      MPI_Send(Ey_snd,ny,MPI_DOUBLE,p,2,MPI_COMM_WORLD);
      MPI_Send(Ey_sndb,npml,MPI_DOUBLE,p,3,MPI_COMM_WORLD);

      MPI_Wait(&rdsync1, &sdsync1);
      MPI_Wait(&rdsync2, &sdsync2);
      MPI_Wait(&rdsync3, &sdsync3);


      // Hz
      for(i=1;i<npml;i++)
	for(j=0;j<npml-1;j++)
	{
	  if(i==npml-1)
	  {
	    Hzx_pc1[i*npml+j] = Dax_pc1[i*npml+j]*Hzx_pc1[i*npml+j] - Dbx_pc1[i*npml+j]*(Ey_rcvu[j] - Ey_pc1[i*npml+j]);
	    Hzy_pc1[i*npml+j] = Dax_pc1[i*npml+j]*Hzy_pc1[i*npml+j] + Dbx_pc1[i*npml+j]*(Ex_pc1[i*(npml+1)+(j+1)] - Ex_pc1[i*(npml+1)+j]);
	  }
	  else
	  {
	    Hzx_pc1[i*npml+j] = Dax_pc1[i*npml+j]*Hzx_pc1[i*npml+j] - Dbx_pc1[i*npml+j]*(Ey_pc1[(i+1)*npml+j] - Ey_pc1[i*npml+j]);
	    Hzy_pc1[i*npml+j] = Dax_pc1[i*npml+j]*Hzy_pc1[i*npml+j] + Dbx_pc1[i*npml+j]*(Ex_pc1[i*(npml+1)+(j+1)] - Ex_pc1[i*(npml+1)+j]);
	  }
	}
      for(i=0;i<npml-1;i++)
	for(j=0;j<npml-1;j++)
	{
	  Hzx_pc2[i*npml+j] = Dax_pc2[i*npml+j]*Hzx_pc2[i*npml+j] - Dbx_pc2[i*npml+j]*(Ey_pc2[(i+1)*npml+j] - Ey_pc2[i*npml+j]);
	  Hzy_pc2[i*npml+j] = Day_pc2[i*npml+j]*Hzy_pc2[i*npml+j] + Dby_pc2[i*npml+j]*(Ex_pc2[i*(npml+1)+j+1] - Ex_pc2[i*(npml+1)+j]);
	}
      for(i=0;i<npml-1;i++)
	for(j=1;j<npml;j++)
	{
	  if(j==npml-1)
	  {
	    Hzx_pc3[i*npml+j] = Dax_pc3[i*npml+j]*Hzx_pc3[i*npml+j] - Dbx_pc3[i*npml+j]*(Ey_pc3[(i+1)*npml+j] - Ey_pc3[i*npml+j]);
	    Hzy_pc3[i*npml+j] = Day_pc3[i*npml+j]*Hzy_pc3[i*npml+j] + Dby_pc3[i*npml+j]*(Ex_ps6[i*ny] - Ex_pc3[i*npml+j]);
	  }
	  else
	  {
	    Hzx_pc3[i*npml+j] = Dax_pc3[i*npml+j]*Hzx_pc3[i*npml+j] - Dbx_pc3[i*npml+j]*(Ey_pc3[(i+1)*npml+j] - Ey_pc3[i*npml+j]);
	    Hzy_pc3[i*npml+j] = Day_pc3[i*npml+j]*Hzy_pc3[i*npml+j] + Dby_pc3[i*npml+j]*(Ex_pc3[i*npml+j+1] - Ex_pc3[i*npml+j]);
	  }
	}
      for(i=1;i<npml;i++)
	for(j=1;j<npml;j++)
	{
	  if(i==npml-1 && j==npml-1)
	  {
	    Hzx_pc4[i*npml+j] = Dax_pc4[i*npml+j]*Hzx_pc4[i*npml+j] - Dbx_pc4[i*npml+j]*(Ey_rcvb[j] - Ey_pc4[i*npml+j]);
	    Hzy_pc4[i*npml+j] = Day_pc4[i*npml+j]*Hzy_pc4[i*npml+j] + Dby_pc4[i*npml+j]*(Ex_ps8[i*ny] - Ex_pc4[i*npml+j]);
	  }
	  else if(i==npml-1)
	  {
	    Hzx_pc4[i*npml+j] = Dax_pc4[i*npml+j]*Hzx_pc4[i*npml+j] - Dbx_pc4[i*npml+j]*(Ey_rcvb[j] - Ey_pc4[i*npml+j]);
	    Hzy_pc4[i*npml+j] = Day_pc4[i*npml+j]*Hzy_pc4[i*npml+j] + Dby_pc4[i*npml+j]*(Ex_pc4[i*npml+j+1] - Ex_pc4[i*npml+j]);
	  }
	  else if(j==npml-1)
	  {
	    Hzx_pc4[i*npml+j] = Dax_pc4[i*npml+j]*Hzx_pc4[i*npml+j] - Dbx_pc4[i*npml+j]*(Ey_pc4[(i+1)*npml+j] - Ey_pc4[i*npml+j]);
	    Hzy_pc4[i*npml+j] = Day_pc4[i*npml+j]*Hzy_pc4[i*npml+j] + Dby_pc4[i*npml+j]*(Ex_ps8[i*ny] - Ex_pc4[i*npml+j]);
	  }
	  else
	  {
	    Hzx_pc4[i*npml+j] = Dax_pc4[i*npml+j]*Hzx_pc4[i*npml+j] - Dbx_pc4[i*npml+j]*(Ey_pc4[(i+1)*npml+j] - Ey_pc4[i*npml+j]);
	    Hzy_pc4[i*npml+j] = Day_pc4[i*npml+j]*Hzy_pc4[i*npml+j] + Dby_pc4[i*npml+j]*(Ex_pc4[i*npml+j+1] - Ex_pc4[i*npml+j]);
	  }
	}
      for(i=0;i<npml-1;i++)
	for(j=0;j<ny;j++)
	{
	  if(j==ny-1)
	  {
	    Hzx_ps6[i*ny+j] = Da_ps6[i*ny+j]*Hzx_ps6[i*ny+j] - Db_ps6[i*ny+j]*(Ey_ps6[(i+1)*ny+j] - Ey_ps6[i*ny+j]);
	    Hzy_ps6[i*ny+j] = Da_ps6[i*ny+j]*Hzy_ps6[i*ny+j] + Db_ps6[i*ny+j]*(Ex_pc2[i*(npml+1)] - Ex_ps6[i*ny+j]);
	  }
	  else
	  {
	    Hzx_ps6[i*ny+j] = Da_ps6[i*ny+j]*Hzx_ps6[i*ny+j] - Db_ps6[i*ny+j]*(Ey_ps6[(i+1)*ny+j] - Ey_ps6[i*ny+j]);
	    Hzy_ps6[i*ny+j] = Da_ps6[i*ny+j]*Hzy_ps6[i*ny+j] + Db_ps6[i*ny+j]*(Ex_ps6[i*ny+j+1] - Ex_ps6[i*ny+j]);
	  }
	}
      for(i=1;i<npml;i++)
	for(j=0;j<ny;j++)
	{
	  if(i==npml-1 && j==ny-1)
	  {
	    Hzx_ps8[i*ny+j] = Da_ps8[i*ny+j]*Hzx_ps8[i*ny+j] - Db_ps8[i*ny+j]*(Ey_rcv[j] - Ey_ps8[i*ny+j]);
	    Hzy_ps8[i*ny+j] = Da_ps8[i*ny+j]*Hzy_ps8[i*ny+j] + Db_ps8[i*ny+j]*(Ex_pc1[i*npml] - Ex_ps8[(i+1)*npml-1]);
	  }
	  else if(i==npml-1)
	  {
	    Hzx_ps8[i*ny+j] = Da_ps8[i*ny+j]*Hzx_ps8[i*ny+j] - Db_ps8[i*ny+j]*(Ey_rcv[j] - Ey_ps8[i*ny+j]);
	    Hzy_ps8[i*ny+j] = Da_ps8[i*ny+j]*Hzy_ps8[i*ny+j] + Db_ps8[i*ny+j]*(Ex_ps8[i*ny+j+1] - Ex_ps8[i*ny+j]);
	  }
	  else if(j==ny-1)
	  {
	    Hzx_ps8[i*ny+j] = Da_ps8[i*ny+j]*Hzx_ps8[i*ny+j] - Db_ps8[i*ny+j]*(Ey_ps8[(i+1)*ny+j] - Ey_ps8[i*ny+j]);
	    Hzy_ps8[i*ny+j] = Da_ps8[i*ny+j]*Hzy_ps8[i*ny+j] + Db_ps8[i*ny+j]*(Ex_pc1[i*(npml+1)] - Ex_ps8[(i+1)*npml-1]);
	  }
	  else
	  {
	    Hzx_ps8[i*ny+j] = Da_ps8[i*ny+j]*Hzx_ps8[i*ny+j] - Db_ps8[i*ny+j]*(Ey_ps8[(i+1)*ny+j] - Ey_ps8[i*ny+j]);
	    Hzy_ps8[i*ny+j] = Da_ps8[i*ny+j]*Hzy_ps8[i*ny+j] + Db_ps8[i*ny+j]*(Ex_ps8[i*ny+j+1] - Ex_ps8[i*ny+j]);
	  }
	}

      // data sync.
      for(i=0;i<npml;i++)
      {
	Hzx_sndu[i] = Hzx_pc1[(npml-1)*npml+i];
	Hzy_sndu[i] = Hzy_pc1[(npml-1)*npml+i];
	Hzx_sndb[i] = Hzx_pc4[(npml-1)*npml+i];
	Hzy_sndb[i] = Hzy_pc4[(npml-1)*npml+i];
      }
      for(i=0;i<ny;i++)
	Hz_snd[i] = Hzx_ps8[(npml-1)*ny+i]+Hzy_ps8[(npml-1)*ny+i];


      MPI_Irecv(Hzx_rcvu,npml,MPI_DOUBLE,p,p*100+4,MPI_COMM_WORLD,&rdsync4);
      MPI_Irecv(Hzy_rcvu,npml,MPI_DOUBLE,p,p*100+5,MPI_COMM_WORLD,&rdsync5);
      MPI_Irecv(Hz_rcv,ny,MPI_DOUBLE,p,p*100+6,MPI_COMM_WORLD,&rdsync8);
      MPI_Irecv(Hzx_rcvb,npml,MPI_DOUBLE,p,p*100+7,MPI_COMM_WORLD,&rdsync6);
      MPI_Irecv(Hzy_rcvb,npml,MPI_DOUBLE,p,p*100+8,MPI_COMM_WORLD,&rdsync7);
      MPI_Send(Hzx_sndu,npml,MPI_DOUBLE,1,4,MPI_COMM_WORLD);
      MPI_Send(Hzy_sndu,npml,MPI_DOUBLE,1,5,MPI_COMM_WORLD);
      MPI_Send(Hz_snd,ny,MPI_DOUBLE,1,6,MPI_COMM_WORLD);
      MPI_Send(Hzx_sndb,npml,MPI_DOUBLE,1,7,MPI_COMM_WORLD);
      MPI_Send(Hzy_sndb,npml,MPI_DOUBLE,1,8,MPI_COMM_WORLD);
      MPI_Wait(&rdsync4, &sdsync4);
      MPI_Wait(&rdsync5, &sdsync5);
      MPI_Wait(&rdsync6, &sdsync6);
      MPI_Wait(&rdsync7, &sdsync7);
      MPI_Wait(&rdsync8, &sdsync8);


      // Ey
      for(i=1;i<npml;i++)
	for(j=0;j<npml-1;j++)
	  Ey_pc1[i*npml+j] = Cax_pc1[i*npml+j]*Ey_pc1[i*npml+j] - Cbx_pc1[i*npml+j]*( (Hzx_pc1[i*npml+j]+Hzy_pc1[i*npml+j] ) - ( Hzx_pc1[(i-1)*npml+j]+Hzy_pc1[(i-1)*npml+j]) );
      for(i=0;i<npml;i++)
	for(j=0;j<npml-1;j++)
	{
	  if(i==0)
	    Ey_pc2[i*npml+j] = Cax_pc2[i*npml+j]*Ey_pc2[i*npml+j] - Cbx_pc2[i*npml+j]*( (Hzx_pc2[i*npml+j]+Hzy_pc2[i*npml+j] ) - (Hzx_rcvu[j]+Hzy_rcvu[j] ) );
	  else
	    Ey_pc2[i*npml+j] = Cax_pc2[i*npml+j]*Ey_pc2[i*npml+j] - Cbx_pc2[i*npml+j]*( (Hzx_pc2[i*npml+j]+Hzy_pc2[i*npml+j] ) - (Hzx_pc2[(i-1)*npml+j]+Hzy_pc2[(i-1)*npml+j] ) );
	}
      for(i=0;i<npml;i++)
	for(j=1;j<npml;j++)
	{
	  if(i==0)
	    Ey_pc3[i*npml+j] = Cax_pc3[i*npml+j]*Ey_pc3[i*npml+j] - Cbx_pc3[i*npml+j]*( (Hzx_pc3[i*npml+j]+Hzy_pc3[i*npml+j] ) - (Hzx_rcvb[j]+Hzy_rcvb[j] ) );
	  else
	    Ey_pc3[i*npml+j] = Cax_pc3[i*npml+j]*Ey_pc3[i*npml+j] - Cbx_pc3[i*npml+j]*( (Hzx_pc3[i*npml+j]+Hzy_pc3[i*npml+j] ) - (Hzx_pc3[(i-1)*npml+j]+Hzy_pc3[(i-1)*npml+j] ) );
	}
      for(i=1;i<npml;i++)
	for(j=1;j<npml;j++)
	  Ey_pc4[i*npml+j] = Cax_pc4[i*npml+j]*Ey_pc4[i*npml+j] - Cbx_pc4[i*npml+j]*( (Hzx_pc4[i*npml+j]+Hzy_pc4[i*npml+j] ) - (Hzx_pc4[(i-1)*npml+j]+Hzy_pc4[(i-1)*npml+j] ) );
      for(i=0;i<npml;i++)
	for(j=0;j<ny;j++)
	{
	  if(i==0)
	    Ey_ps6[i*ny+j] = Ca_ps6[i*ny+j]*Ey_ps6[i*ny+j] - Cb_ps6[i*ny+j]*( (Hzx_ps6[i*ny+j]+Hzy_ps6[i*ny+j] ) - Hz_rcv[j] );
	  else
	    Ey_ps6[i*ny+j] = Ca_ps6[i*ny+j]*Ey_ps6[i*ny+j] - Cb_ps6[i*ny+j]*( (Hzx_ps6[i*ny+j]+Hzy_ps6[i*ny+j] ) - (Hzx_ps6[(i-1)*ny+j]+Hzy_ps6[(i-1)*ny+j] ) );
	}
      for(i=1;i<npml;i++)
	for(j=0;j<ny;j++)
	  Ey_ps8[i*ny+j] = Ca_ps8[i*ny+j]*Ey_ps8[i*ny+j] - Cb_ps8[i*ny+j]*( (Hzx_ps8[i*ny+j]+Hzy_ps8[i*ny+j] ) - (Hzx_ps8[(i-1)*ny+j]+Hzy_ps8[(i-1)*ny+j] ) );



      // Ex
      for(i=1;i<npml;i++)
	for(j=0;j<npml;j++)
	{
	  if(j==0)
	    Ex_pc1[i*(npml+1)+j] = Cay_pc1[i*npml+j]*Ex_pc1[i*(npml+1)+j] + Cby_pc1[i*npml+j]*( ( Hzx_pc1[i*npml+j]+Hzy_pc1[i*npml+j] ) - ( Hzx_ps8[(i+1)*ny-1]+Hzy_ps8[(i+1)*ny-1]) );
	  else
	    Ex_pc1[i*(npml+1)+j] = Cay_pc1[i*npml+j]*Ex_pc1[i*(npml+1)+j] + Cby_pc1[i*npml+j]*( ( Hzx_pc1[i*npml+j]+Hzy_pc1[i*npml+j] ) - ( Hzx_pc1[i*npml+(j-1)]+Hzy_pc1[i*npml+(j-1)]) );
	}
      for(i=0;i<npml-1;i++)
	for(j=0;j<npml;j++)
	{
	  if(j==0)
	    Ex_pc2[i*(npml+1)+j] = Cay_pc2[i*npml+j]*Ex_pc2[i*(npml+1)+j] + Cby_pc2[i*npml+j]*( ( Hzx_pc2[i*npml+j]+Hzy_pc2[i*npml+j] ) - ( Hzx_ps6[(i+1)*ny-1]+Hzy_ps6[(i+1)*ny-1]) );
	  else
	    Ex_pc2[i*(npml+1)+j] = Cay_pc2[i*npml+j]*Ex_pc2[i*(npml+1)+j] + Cby_pc2[i*npml+j]*( ( Hzx_pc2[i*npml+j]+Hzy_pc2[i*npml+j] ) - ( Hzx_pc2[i*npml+(j-1)]+Hzy_pc2[i*npml+(j-1)]) );
	}
      for(i=0;i<npml-1;i++)
	for(j=1;j<npml;j++)
	  Ex_pc3[i*npml+j] = Cay_pc3[i*npml+j]*Ex_pc3[i*npml+j] + Cby_pc3[i*npml+j]*( ( Hzx_pc3[i*npml+j]+Hzy_pc3[i*npml+j] )-(Hzx_pc3[i*npml+j-1]+Hzy_pc3[i*npml+j-1]) );
      for(i=1;i<npml;i++)
	for(j=1;j<npml;j++)
	  Ex_pc4[i*npml+j] = Cay_pc4[i*npml+j]*Ex_pc4[i*npml+j] + Cby_pc4[i*npml+j]*( ( Hzx_pc4[i*npml+j]+Hzy_pc4[i*npml+j] )-(Hzx_pc4[i*npml+j-1]+Hzy_pc4[i*npml+j-1]) );
      for(i=0;i<npml-1;i++)
	for(j=0;j<ny;j++)
	{
	  if(j==0)
	    Ex_ps6[i*ny+j] = Ca_ps6[i*ny+j]*Ex_ps6[i*ny+j] + Cb_ps6[i*ny+j]*( ( Hzx_ps6[i*ny+j]+Hzy_ps6[i*ny+j] )-( Hzx_pc3[(i+1)*npml-1]+Hzy_pc3[(i+1)*npml-1]) );
	  else
	    Ex_ps6[i*ny+j] = Ca_ps6[i*ny+j]*Ex_ps6[i*ny+j] + Cb_ps6[i*ny+j]*( ( Hzx_ps6[i*ny+j]+Hzy_ps6[i*ny+j] )-( Hzx_ps6[i*ny+(j-1)]+Hzy_ps6[i*ny+(j-1)]) );
	}
      for(i=1;i<npml;i++)
	for(j=0;j<ny;j++)
	{
	  if(j==0)
	    Ex_ps8[i*ny+j] = Ca_ps8[i*ny+j]*Ex_ps8[i*ny+j] + Cb_ps8[i*ny+j]*( ( Hzx_ps8[i*ny+j]+Hzy_ps8[i*ny+j] )-(Hzx_pc4[(i+1)*npml-1]+Hzy_pc4[(i+1)*npml-1]) );
	  else
	    Ex_ps8[i*ny+j] = Ca_ps8[i*ny+j]*Ex_ps8[i*ny+j] + Cb_ps8[i*ny+j]*( ( Hzx_ps8[i*ny+j]+Hzy_ps8[i*ny+j] )-(Hzx_ps8[i*ny+j-1]+Hzy_ps8[i*ny+j-1]) );
	}

      // Save total field

      /*
	 if(iteration>=2000)
	 {
	 MPI_Irecv(N1,nx*ny/p,MPI_DOUBLE,1,1,MPI_COMM_WORLD,&request1);
	 MPI_Irecv(N2,nx*ny/p,MPI_DOUBLE,2,2,MPI_COMM_WORLD,&request2);
	 MPI_Irecv(N3,nx*ny/p,MPI_DOUBLE,3,3,MPI_COMM_WORLD,&request3);
	 MPI_Irecv(N4,nx*ny/p,MPI_DOUBLE,4,4,MPI_COMM_WORLD,&request4);
	 MPI_Irecv(N5,nx*ny/p,MPI_DOUBLE,5,5,MPI_COMM_WORLD,&request5);
	 MPI_Irecv(N6,nx*ny/p,MPI_DOUBLE,6,6,MPI_COMM_WORLD,&request6);
	 MPI_Irecv(N7,nx*ny/p,MPI_DOUBLE,7,7,MPI_COMM_WORLD,&request7);
	 MPI_Irecv(N8,nx*ny/p,MPI_DOUBLE,8,8,MPI_COMM_WORLD,&request8);
	 MPI_Irecv(N9,nx*ny/p,MPI_DOUBLE,9,9,MPI_COMM_WORLD,&request9);
	 MPI_Irecv(N10,nx*ny/p,MPI_DOUBLE,10,10,MPI_COMM_WORLD,&request10);
	 MPI_Irecv(N11,nx*ny/p,MPI_DOUBLE,11,11,MPI_COMM_WORLD,&request11);
	 MPI_Irecv(N12,nx*ny/p,MPI_DOUBLE,12,12,MPI_COMM_WORLD,&request12);
	 MPI_Irecv(N13,nx*ny/p,MPI_DOUBLE,13,13,MPI_COMM_WORLD,&request13);
	 MPI_Irecv(N14,nx*ny/p,MPI_DOUBLE,14,14,MPI_COMM_WORLD,&request14);
	 MPI_Irecv(N15,nx*ny/p,MPI_DOUBLE,15,15,MPI_COMM_WORLD,&request15);
	 MPI_Irecv(N16,nx*ny/p,MPI_DOUBLE,16,16,MPI_COMM_WORLD,&request16);

	 MPI_Wait(&request1, &status1);
	 MPI_Wait(&request2, &status2);
	 MPI_Wait(&request3, &status3);
	 MPI_Wait(&request4, &status4);
	 MPI_Wait(&request5, &status5);
	 MPI_Wait(&request6, &status6);
	 MPI_Wait(&request7, &status7);
	 MPI_Wait(&request8, &status8);
	 MPI_Wait(&request9, &status9);
	 MPI_Wait(&request10, &status10);
	 MPI_Wait(&request11, &status11);
	 MPI_Wait(&request12, &status12);
	 MPI_Wait(&request13, &status13);
	 MPI_Wait(&request14, &status14);
	 MPI_Wait(&request15, &status15);
	 MPI_Wait(&request16, &status16);

      // Sync. NODE 1
      for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
      total[i*ny+j]+=N1[i*ny+j];
      // Sync. NODE 2
      for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
      total[(nx/p+i)*ny+j]+=N2[i*ny+j];
      // Sync. NODE 3
      for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
      total[(2*nx/p+i)*ny+j]+=N3[i*ny+j];
      // Sync. NODE 4
      for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
      total[(3*nx/p+i)*ny+j]+=N4[i*ny+j];
      // Sync. NODE 5
      for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
      total[(4*nx/p+i)*ny+j]+=N5[i*ny+j];
      // Sync. NODE 6
      for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
      total[(5*nx/p+i)*ny+j]+=N6[i*ny+j];
      // Sync. NODE 7
      for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
      total[(6*nx/p+i)*ny+j]+=N7[i*ny+j];
      // Sync. NODE 8
      for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
      total[(7*nx/p+i)*ny+j]+=N8[i*ny+j];
      // Sync. NODE 9
      for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
      total[(8*nx/p+i)*ny+j]+=N9[i*ny+j];
      // Sync. NODE 10
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	  total[(9*nx/p+i)*ny+j]+=N10[i*ny+j];
      // Sync. NODE 11
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	  total[(10*nx/p+i)*ny+j]+=N11[i*ny+j];
      // Sync. NODE 12
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	  total[(11*nx/p+i)*ny+j]+=N12[i*ny+j];
      // Sync. NODE 13
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	  total[(12*nx/p+i)*ny+j]+=N13[i*ny+j];
      // Sync. NODE 14
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	  total[(13*nx/p+i)*ny+j]+=N14[i*ny+j];
      // Sync. NODE 15
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	  total[(14*nx/p+i)*ny+j]+=N15[i*ny+j];
      // Sync. NODE 16
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	  total[(15*nx/p+i)*ny+j]+=N16[i*ny+j];


      if(iteration==2999)
      {
	sprintf(pic_file,"time_avg.dat");
	output_file=fopen(pic_file,"w");
	for(j=0;j<ny;j++)
	{
	  for(i=450;i<nx;i++)
	    fprintf(output_file,"\t%e",total[i*ny+j]*total[i*ny+j]/1000.0);
	  fprintf(output_file,"\n");
	}

	fclose(output_file);
      }
    } // if(iteration%5==0)
    */

    } // end of iteration

    // Save total field

    MPI_Irecv(N1,nx*ny/p,MPI_DOUBLE,1,1,MPI_COMM_WORLD,&request1);
    MPI_Irecv(N2,nx*ny/p,MPI_DOUBLE,2,2,MPI_COMM_WORLD,&request2);
    MPI_Irecv(N3,nx*ny/p,MPI_DOUBLE,3,3,MPI_COMM_WORLD,&request3);
    MPI_Irecv(N4,nx*ny/p,MPI_DOUBLE,4,4,MPI_COMM_WORLD,&request4);
    MPI_Irecv(N5,nx*ny/p,MPI_DOUBLE,5,5,MPI_COMM_WORLD,&request5);
    MPI_Irecv(N6,nx*ny/p,MPI_DOUBLE,6,6,MPI_COMM_WORLD,&request6);
    MPI_Irecv(N7,nx*ny/p,MPI_DOUBLE,7,7,MPI_COMM_WORLD,&request7);
    MPI_Irecv(N8,nx*ny/p,MPI_DOUBLE,8,8,MPI_COMM_WORLD,&request8);
    MPI_Irecv(N9,nx*ny/p,MPI_DOUBLE,9,9,MPI_COMM_WORLD,&request9);
    MPI_Irecv(N10,nx*ny/p,MPI_DOUBLE,10,10,MPI_COMM_WORLD,&request10);
    MPI_Irecv(N11,nx*ny/p,MPI_DOUBLE,11,11,MPI_COMM_WORLD,&request11);
    MPI_Irecv(N12,nx*ny/p,MPI_DOUBLE,12,12,MPI_COMM_WORLD,&request12);
    /*
       MPI_Irecv(N13,nx*ny/p,MPI_DOUBLE,13,13,MPI_COMM_WORLD,&request13);
       MPI_Irecv(N14,nx*ny/p,MPI_DOUBLE,14,14,MPI_COMM_WORLD,&request14);
       MPI_Irecv(N15,nx*ny/p,MPI_DOUBLE,15,15,MPI_COMM_WORLD,&request15);
       MPI_Irecv(N16,nx*ny/p,MPI_DOUBLE,16,16,MPI_COMM_WORLD,&request16);
       */

    MPI_Wait(&request1, &status1);
    MPI_Wait(&request2, &status2);
    MPI_Wait(&request3, &status3);
    MPI_Wait(&request4, &status4);
    MPI_Wait(&request5, &status5);
    MPI_Wait(&request6, &status6);
    MPI_Wait(&request7, &status7);
    MPI_Wait(&request8, &status8);
    MPI_Wait(&request9, &status9);
    MPI_Wait(&request10, &status10);
    MPI_Wait(&request11, &status11);
    MPI_Wait(&request12, &status12);
    /*
       MPI_Wait(&request13, &status13);
       MPI_Wait(&request14, &status14);
       MPI_Wait(&request15, &status15);
       MPI_Wait(&request16, &status16);
       */

    // Sync. NODE 1
    for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
	total[i*ny+j]=N1[i*ny+j];
    // Sync. NODE 2
    for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
	total[(nx/p+i)*ny+j]=N2[i*ny+j];
    // Sync. NODE 3
    for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
	total[(2*nx/p+i)*ny+j]=N3[i*ny+j];
    // Sync. NODE 4
    for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
	total[(3*nx/p+i)*ny+j]=N4[i*ny+j];
    // Sync. NODE 5
    for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
	total[(4*nx/p+i)*ny+j]=N5[i*ny+j];
    // Sync. NODE 6
    for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
	total[(5*nx/p+i)*ny+j]=N6[i*ny+j];
    // Sync. NODE 7
    for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
	total[(6*nx/p+i)*ny+j]=N7[i*ny+j];
    // Sync. NODE 8
    for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
	total[(7*nx/p+i)*ny+j]=N8[i*ny+j];
    // Sync. NODE 9
    for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
	total[(8*nx/p+i)*ny+j]=N9[i*ny+j];
    // Sync. NODE 10
    for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
	total[(9*nx/p+i)*ny+j]=N10[i*ny+j];
    // Sync. NODE 11
    for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
	total[(10*nx/p+i)*ny+j]=N11[i*ny+j];
    // Sync. NODE 12
    for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
	total[(11*nx/p+i)*ny+j]=N12[i*ny+j];
    /*
    // Sync. NODE 13
    for(i=0;i<nx/p;i++)
    for(j=0;j<ny;j++)
    total[(12*nx/p+i)*ny+j]=N13[i*ny+j];
    // Sync. NODE 14
    for(i=0;i<nx/p;i++)
    for(j=0;j<ny;j++)
    total[(13*nx/p+i)*ny+j]=N14[i*ny+j];
    // Sync. NODE 15
    for(i=0;i<nx/p;i++)
    for(j=0;j<ny;j++)
    total[(14*nx/p+i)*ny+j]=N15[i*ny+j];
    // Sync. NODE 16
    for(i=0;i<nx/p;i++)
    for(j=0;j<ny;j++)
    total[(15*nx/p+i)*ny+j]=N16[i*ny+j];
    */



    // Making a output file in VTK Data file format version 2.0
    output_file=fopen("Z-field.dat","w");
    for(j=0;j<ny;j++)
    {
      for(i=0;i<nx;i++)
	fprintf(output_file,"\t%.2e",total[i*ny+j]);
      fprintf(output_file,"\n");
    }
    fclose(output_file);
    output_file=fopen("Z-energy.dat","w");
    for(j=0;j<ny;j++)
    {
      for(i=0;i<nx;i++)
	fprintf(output_file,"\t%.2e",total[i*ny+j]*total[i*ny+j]);
      fprintf(output_file,"\n");
    }
    fclose(output_file);


  }// if(node ==0)

  ///////////////////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////////////////
  ////////////////////////////////     NODE p START    /////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////////////////
  else if(node == p)
  { 

    // PML 5-16,7-16
    double *Ca_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Cb_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Da_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Db_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml));

    double *Ca_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Cb_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Da_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Db_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));

    // FS
    double *Cb = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Cc = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Dc = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Db = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Fx = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Fy = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Fz = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Gamma_e = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Gamma_h = (double *)malloc(sizeof(double)*(nx/p)*(ny));

    // current source
    double *M = (double *)malloc(sizeof(double)*(nx/p)*(ny));

    // Fields
    double *Ex = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Ey = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Hz = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Hz_dmy = (double *)malloc(sizeof(double)*(nx/p)*(ny));

    double *Ex_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml+1));
    double *Ey_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Hzx_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Hzy_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml));

    double *Ex_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Ey_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Hzx_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Hzy_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));

    double *sigma_p = (double *)malloc(sizeof(double)*npml);

    // shared components
    // send
    double *Ey_sndu = (double *)malloc(sizeof(double)*npml);
    double *Ey_snd = (double *)malloc(sizeof(double)*ny);
    double *Ey_sndb = (double *)malloc(sizeof(double)*npml);
    double *Hzx_sndu = (double *)malloc(sizeof(double)*npml);
    double *Hzy_sndu = (double *)malloc(sizeof(double)*npml);
    double *Hz_snd = (double *)malloc(sizeof(double)*ny);
    double *Hzx_sndb = (double *)malloc(sizeof(double)*npml);
    double *Hzy_sndb = (double *)malloc(sizeof(double)*npml);

    // receive
    double *Ey_rcvu = (double *)malloc(sizeof(double)*npml);
    double *Ey_rcv = (double *)malloc(sizeof(double)*ny);
    double *Ey_rcvb = (double *)malloc(sizeof(double)*npml);
    double *Hzx_rcvu = (double *)malloc(sizeof(double)*npml);
    double *Hzy_rcvu = (double *)malloc(sizeof(double)*npml);
    double *Hz_rcv = (double *)malloc(sizeof(double)*ny);
    double *Hzx_rcvb = (double *)malloc(sizeof(double)*npml);
    double *Hzy_rcvb = (double *)malloc(sizeof(double)*npml);

    FILE *system;
    char filename[256];

    for(i=0;i<ny;i++)
    {
      if(i<npml)
      {
	Ey_sndu[i] = 0.0;
	Ey_snd[i] = 0.0;
	Ey_sndb[i] = 0.0;
	Hzx_sndu[i] = 0.0;
	Hzy_sndu[i] = 0.0;
	Hz_snd[i] = 0.0;
	Hzx_sndb[i] = 0.0;
	Hzy_sndb[i] = 0.0;
	Ey_rcvu[i] = 0.0;
	Ey_rcv[i] = 0.0;
	Ey_rcvb[i] = 0.0;
	Hzx_rcvu[i] = 0.0;
	Hzy_rcvu[i] = 0.0;
	Hz_rcv[i] = 0.0;
	Hzx_rcvb[i] = 0.0;
	Hzy_rcvb[i] = 0.0;
      }
      else
      {
	Ey_snd[i] = 0.0;
	Hz_snd[i] = 0.0;
	Ey_rcv[i] = 0.0;
	Hz_rcv[i] = 0.0;
      }
    }

    for(i=0;i<npml;i++) 
      sigma_p[i] = (double)((double)sigma_m*(double)((i+1)*(i+1)*(i+1)))/(double)(npml*npml*npml);
    sigma_p[0] = ((sigma_p[0] - 0)/2.0 + sigma_p[0] + (sigma_p[0] + sigma_p[1])/2.0)/3.0;
    for(i=1;i<npml-1;i++) 
      sigma_p[i] = ((sigma_p[i-1]+sigma_p[i])/2.0 + sigma_p[i] + (sigma_p[i] + sigma_p[i+1])/2.0)/3.0;

    for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
      {
	if(j<npml+1)
	{
	  if(j==npml)
	    Ex_psu[i*(npml+1)+j] = 0.0;
	  else
	  {
	    Ex_psu[i*(npml+1)+j] = 0.0;
	    Ey_psu[i*npml+j] = 0.0;
	    Hzx_psu[i*npml+j] = 0.0;
	    Hzy_psu[i*npml+j] = 0.0;

	    Ex_psb[i*npml+j] = 0.0;
	    Ey_psb[i*npml+j] = 0.0;
	    Hzx_psb[i*npml+j] = 0.0;
	    Hzy_psb[i*npml+j] = 0.0;
	  }
	}
	Ex[i*ny+j] = 0.0;
	Ey[i*ny+j] = 0.0;
	Hz[i*ny+j] = 0.0;
	Hz_dmy[i*ny+j] = 0.0;
	M[i*ny+j] = 0.0;
      }

    sprintf(filename,"node%d.dat",node);
    system = fopen(filename,"r");

    if(strcmp(source_mode,"TE")==0)
    {
      for(i=0;i<nx/p;i++) 
	for(j=0;j<npml;j++) 
	{ 
	  Ca_psu[i*npml+j] = (1-sigma_p[j]*dT/(2*EPSILON_0))/(1+sigma_p[j]*dT/(2*EPSILON_0)); 
	  Cb_psu[i*npml+j] = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[j]*dT/(2*EPSILON_0)); 
	  Da_psu[i*npml+j] = (1-SH*sigma_p[j]*dT/(2*MU_0))/(1+SH*sigma_p[j]*dT/(2*MU_0)); 
	  Db_psu[i*npml+j] = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[j]*dT/(2*MU_0)); 

	  Ca_psb[i*npml+j] = (1-sigma_p[npml-1-j]*dT/(2*EPSILON_0))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0)); 
	  Cb_psb[i*npml+j] = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0)); 
	  Da_psb[i*npml+j] = (1-SH*sigma_p[npml-1-j]*dT/(2*MU_0))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0)); 
	  Db_psb[i*npml+j] = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0)); 
	}

      for(j=0;j<ny;j++) 
	for(i=0;i<nx/p;i++) 
	{ 
	  fscanf(system,"%d",&index);
	  Cb[i*ny+j] = sqrt(EPSILON_0/MU_0)*(dT/(materials[index].permitivity*cellsize));
	  Cc[i*ny+j] = dT*materials[index].p_freq_elec*materials[index].p_freq_elec;
	  Db[i*ny+j] = sqrt(MU_0/EPSILON_0)*(dT/(materials[index].permiability*cellsize));
	  Dc[i*ny+j] = dT*materials[index].p_freq_mag*materials[index].p_freq_mag;
	  Fx[i*ny+j] = 0.0;
	  Fy[i*ny+j] = 0.0;
	  Fz[i*ny+j] = 0.0;
	  Gamma_e[i*ny+j] = materials[index].r_rate_elec;
	  Gamma_h[i*ny+j] = materials[index].r_rate_mag;
	}
    }// end of if(strcmp(source_mode,"TE")==0)

    if(strcmp(source_mode,"TM")==0)
    {
      for(i=0;i<nx/p;i++) 
	for(j=0;j<npml;j++) 
	{ 
	  Da_psu[i*npml+j] = (1-sigma_p[j]*dT/(2*EPSILON_0))/(1+sigma_p[j]*dT/(2*EPSILON_0)); 
	  Db_psu[i*npml+j] = -sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[j]*dT/(2*EPSILON_0)); 
	  Ca_psu[i*npml+j] = (1-SH*sigma_p[j]*dT/(2*MU_0))/(1+SH*sigma_p[j]*dT/(2*MU_0)); 
	  Cb_psu[i*npml+j] = -sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[j]*dT/(2*MU_0)); 

	  Da_psb[i*npml+j] = (1-sigma_p[npml-1-j]*dT/(2*EPSILON_0))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0)); 
	  Db_psb[i*npml+j] = -sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0)); 
	  Ca_psb[i*npml+j] = (1-SH*sigma_p[npml-1-j]*dT/(2*MU_0))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0)); 
	  Cb_psb[i*npml+j] = -sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0)); 
	}

      for(j=0;j<ny;j++) 
	for(i=0;i<nx/p;i++) 
	{ 
	  fscanf(system,"%d",&index);
	  Db[i*ny+j] = -sqrt(EPSILON_0/MU_0)*(dT/(materials[index].permitivity*cellsize));
	  Dc[i*ny+j] = dT*materials[index].p_freq_elec*materials[index].p_freq_elec;
	  Cb[i*ny+j] = -sqrt(MU_0/EPSILON_0)*(dT/(materials[index].permiability*cellsize));
	  Cc[i*ny+j] = dT*materials[index].p_freq_mag*materials[index].p_freq_mag;
	  Fx[i*ny+j] = 0.0;
	  Fy[i*ny+j] = 0.0;
	  Fz[i*ny+j] = 0.0;
	  Gamma_e[i*ny+j] = materials[index].r_rate_elec;
	  Gamma_h[i*ny+j] = materials[index].r_rate_mag;
	}
    }// end of if(strcmp(source_mode,"TM")==0)
    fclose(system);

    // fields update
    for(iteration=0;iteration<MAXTIMESTEP;iteration++)
    {
      // Current source input
      if(Bp==node)
      {
	for(j=0;j<ny;j++)
	{
	  M[source_xp*ny+j] = exp(-pow(((double)(j-source_yp))/(double)w_of_beam,2))*creal(cexp(-I*(2.0*M_PI*freq*sin(theta)*(double)(j-source_yp)*cellsize/LIGHT_SPEED - 2.0*M_PI*freq*(double)iteration*dT)));
	}
      }
      // data sync.
      for(i=0;i<npml;i++)
      {
	Ey_sndu[i] = Ey_psu[i];
	Ey_sndb[i] = Ey_psb[i];
      }
      for(i=0;i<ny;i++)
	Ey_snd[i] = Ey[i];

      MPI_Irecv(Ey_rcvu,npml,MPI_DOUBLE,0,1,MPI_COMM_WORLD,&rdsync1);
      MPI_Irecv(Ey_rcv,ny,MPI_DOUBLE,0,2,MPI_COMM_WORLD,&rdsync2);
      MPI_Irecv(Ey_rcvb,npml,MPI_DOUBLE,0,3,MPI_COMM_WORLD,&rdsync3);

      MPI_Send(Ey_sndu,npml,MPI_DOUBLE,node-1,node*100+1,MPI_COMM_WORLD);
      MPI_Send(Ey_snd,ny,MPI_DOUBLE,node-1,node*100+2,MPI_COMM_WORLD);
      MPI_Send(Ey_sndb,npml,MPI_DOUBLE,node-1,node*100+3,MPI_COMM_WORLD);

      MPI_Wait(&rdsync1, &sdsync1);
      MPI_Wait(&rdsync2, &sdsync2);
      MPI_Wait(&rdsync3, &sdsync3);


      // Dispersion function
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	  Fz[i*ny+j] = dT*Hz[i*ny+j] + exp(-Gamma_h[i*ny+j]*dT)*Fz[i*ny+j];

      // Hz
      for(i=0;i<nx/p;i++)
	for(j=0;j<npml-1;j++)
	{
	  if(i==nx/p-1)
	  {
	    Hzx_psu[i*npml+j] = Da_psu[i*npml+j]*Hzx_psu[i*npml+j] - Db_psu[i*npml+j]*(Ey_rcvu[j] - Ey_psu[i*npml+j]);
	    Hzy_psu[i*npml+j] = Da_psu[i*npml+j]*Hzy_psu[i*npml+j] + Db_psu[i*npml+j]*(Ex_psu[i*(npml+1)+(j+1)] - Ex_psu[i*(npml+1)+j]);
	  }
	  else
	  {
	    Hzx_psu[i*npml+j] = Da_psu[i*npml+j]*Hzx_psu[i*npml+j] - Db_psu[i*npml+j]*(Ey_psu[(i+1)*npml+j] - Ey_psu[i*npml+j]);
	    Hzy_psu[i*npml+j] = Da_psu[i*npml+j]*Hzy_psu[i*npml+j] + Db_psu[i*npml+j]*(Ex_psu[i*(npml+1)+(j+1)] - Ex_psu[i*(npml+1)+j]);
	  }
	}
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	{
	  if(i==nx/p-1 && j==ny-1)
	    Hz[i*ny+j] = Hz[i*ny+j] + Db[i*ny+j]*(Ex_psu[i*(npml+1)] - Ex[i*ny+j] - Ey_rcv[j] + Ey[i*ny+j] - cellsize*M[i*ny+j]) - Dc[i*ny+j]*Fz[i*ny+j];
	  else if(i==nx/p-1)
	    Hz[i*ny+j] = Hz[i*ny+j] + Db[i*ny+j]*(Ex[i*ny+(j+1)] - Ex[i*ny+j] - Ey_rcv[j] + Ey[i*ny+j] - cellsize*M[i*ny+j]) - Dc[i*ny+j]*Fz[i*ny+j];
	  else if(j==ny-1)
	    Hz[i*ny+j] = Hz[i*ny+j] + Db[i*ny+j]*(Ex_psu[i*(npml+1)] - Ex[i*ny+j] - Ey[(i+1)*ny+j] + Ey[i*ny+j] - cellsize*M[i*ny+j]) - Dc[i*ny+j]*Fz[i*ny+j];
	  else
	    Hz[i*ny+j] = Hz[i*ny+j] + Db[i*ny+j]*(Ex[i*ny+(j+1)] - Ex[i*ny+j] - Ey[(i+1)*ny+j] + Ey[i*ny+j] - cellsize*M[i*ny+j]) - Dc[i*ny+j]*Fz[i*ny+j];
	}
      for(i=0;i<nx/p;i++)
	for(j=1;j<npml;j++)
	{
	  if(i==nx/p-1 && j==npml-1)
	  {
	    Hzx_psb[i*npml+j] = Da_psb[i*npml+j]*Hzx_psb[i*npml+j] - Db_psb[i*npml+j]*(Ey_rcvb[j] - Ey_psb[i*npml+j]);
	    Hzy_psb[i*npml+j] = Da_psb[i*npml+j]*Hzy_psb[i*npml+j] + Db_psb[i*npml+j]*(Ex[i*ny] - Ex_psb[i*npml+j]);
	  }
	  else if(i==nx/p-1)
	  {
	    Hzx_psb[i*npml+j] = Da_psb[i*npml+j]*Hzx_psb[i*npml+j] - Db_psb[i*npml+j]*(Ey_rcvb[j] - Ey_psb[i*npml+j]);
	    Hzy_psb[i*npml+j] = Da_psb[i*npml+j]*Hzy_psb[i*npml+j] + Db_psb[i*npml+j]*(Ex_psb[i*npml+j+1] - Ex_psb[i*npml+j]);
	  }
	  else if(j==npml-1)
	  {
	    Hzx_psb[i*npml+j] = Da_psb[i*npml+j]*Hzx_psb[i*npml+j] - Db_psb[i*npml+j]*(Ey_psb[(i+1)*npml+j] - Ey_psb[i*npml+j]);
	    Hzy_psb[i*npml+j] = Da_psb[i*npml+j]*Hzy_psb[i*npml+j] + Db_psb[i*npml+j]*(Ex[i*ny] - Ex_psb[i*npml+j]);
	  }
	  else
	  {
	    Hzx_psb[i*npml+j] = Da_psb[i*npml+j]*Hzx_psb[i*npml+j] - Db_psb[i*npml+j]*(Ey_psb[(i+1)*npml+j] - Ey_psb[i*npml+j]);
	    Hzy_psb[i*npml+j] = Da_psb[i*npml+j]*Hzy_psb[i*npml+j] + Db_psb[i*npml+j]*(Ex_psb[i*npml+j+1] - Ex_psb[i*npml+j]);
	  }
	}
      // data sync.
      for(i=0;i<npml;i++)
      {
	Hzx_sndu[i] = Hzx_psu[(nx/p-1)*npml+i];
	Hzy_sndu[i] = Hzy_psu[(nx/p-1)*npml+i];
	Hzx_sndb[i] = Hzx_psb[(nx/p-1)*npml+i];
	Hzy_sndb[i] = Hzy_psb[(nx/p-1)*npml+i];
      }
      for(i=0;i<ny;i++)
	Hz_snd[i] = Hz[(nx/p-1)*ny+i];

      MPI_Irecv(Hzx_rcvu,npml,MPI_DOUBLE,node-1,(node-1)*100+4,MPI_COMM_WORLD,&rdsync4);
      MPI_Irecv(Hzy_rcvu,npml,MPI_DOUBLE,node-1,(node-1)*100+5,MPI_COMM_WORLD,&rdsync5);
      MPI_Irecv(Hz_rcv,ny,MPI_DOUBLE,node-1,(node-1)*100+6,MPI_COMM_WORLD,&rdsync6);
      MPI_Irecv(Hzx_rcvb,npml,MPI_DOUBLE,node-1,(node-1)*100+7,MPI_COMM_WORLD,&rdsync7);
      MPI_Irecv(Hzy_rcvb,npml,MPI_DOUBLE,node-1,(node-1)*100+8,MPI_COMM_WORLD,&rdsync8);

      MPI_Send(Hzx_sndu,npml,MPI_DOUBLE,0,node*100+4,MPI_COMM_WORLD);
      MPI_Send(Hzy_sndu,npml,MPI_DOUBLE,0,node*100+5,MPI_COMM_WORLD);
      MPI_Send(Hz_snd,ny,MPI_DOUBLE,0,node*100+6,MPI_COMM_WORLD);
      MPI_Send(Hzx_sndb,npml,MPI_DOUBLE,0,node*100+7,MPI_COMM_WORLD);
      MPI_Send(Hzy_sndb,npml,MPI_DOUBLE,0,node*100+8,MPI_COMM_WORLD);

      MPI_Wait(&rdsync4, &sdsync4);
      MPI_Wait(&rdsync5, &sdsync5);
      MPI_Wait(&rdsync6, &sdsync6);
      MPI_Wait(&rdsync7, &sdsync7);
      MPI_Wait(&rdsync8, &sdsync8);


      // Dispersion function
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	  Fy[i*ny+j] = dT*Ey[i*ny+j] + exp(-Gamma_e[i*ny+j]*dT)*Fy[i*ny+j];

      // Ey
      for(i=0;i<nx/p;i++)
	for(j=0;j<npml-1;j++)
	{
	  if(i==0)
	    Ey_psu[i*npml+j] = Ca_psu[i*npml+j]*Ey_psu[i*npml+j] - Cb_psu[i*npml+j]*( (Hzx_psu[i*npml+j]+Hzy_psu[i*npml+j] ) - ( Hzx_rcvu[j]+Hzy_rcvu[j]) );
	  else
	    Ey_psu[i*npml+j] = Ca_psu[i*npml+j]*Ey_psu[i*npml+j] - Cb_psu[i*npml+j]*( (Hzx_psu[i*npml+j]+Hzy_psu[i*npml+j] ) - ( Hzx_psu[(i-1)*npml+j]+Hzy_psu[(i-1)*npml+j]) );
	}
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	{
	  if(i==0)
	    Ey[i*ny+j] = Ey[i*ny+j] - Cb[i*ny+j]*(Hz[i*ny+j] - Hz_rcv[j]) - Cc[i*ny+j]*Fy[i*ny+j];
	  else
	    Ey[i*ny+j] = Ey[i*ny+j] - Cb[i*ny+j]*(Hz[i*ny+j] - Hz[(i-1)*ny+j]) - Cc[i*ny+j]*Fy[i*ny+j];
	}
      for(i=0;i<nx/p;i++)
	for(j=1;j<npml-1;j++)
	{
	  if(i==0)
	    Ey_psb[i*npml+j] = Ca_psb[i*npml+j]*Ey_psb[i*npml+j] - Cb_psb[i*npml+j]*( (Hzx_psb[i*npml+j]+Hzy_psb[i*npml+j] ) - (Hzx_rcvb[j]+Hzy_rcvb[j] ) );
	  else
	    Ey_psb[i*npml+j] = Ca_psb[i*npml+j]*Ey_psb[i*npml+j] - Cb_psb[i*npml+j]*( (Hzx_psb[i*npml+j]+Hzy_psb[i*npml+j] ) - (Hzx_psb[(i-1)*npml+j]+Hzy_psb[(i-1)*npml+j] ) );
	}

      // Dispersion function
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	  Fx[i*ny+j] = dT*Ex[i*ny+j] + exp(-Gamma_e[i*ny+j]*dT)*Fx[i*ny+j];

      // Ex
      for(i=0;i<nx/p;i++)
	for(j=0;j<npml;j++)
	{
	  if(j==0)
	    Ex_psu[i*(npml+1)+j] = Ca_psu[i*npml+j]*Ex_psu[i*(npml+1)+j] + Cb_psu[i*npml+j]*( ( Hzx_psu[i*npml+j]+Hzy_psu[i*npml+j] ) - ( Hz[(i+1)*ny-1] ) );
	  else
	    Ex_psu[i*(npml+1)+j] = Ca_psu[i*npml+j]*Ex_psu[i*(npml+1)+j] + Cb_psu[i*npml+j]*( ( Hzx_psu[i*npml+j]+Hzy_psu[i*npml+j] ) - ( Hzx_psu[i*npml+j-1]+Hzy_psu[i*npml+j-1] ) );
	}
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	{
	  if(j==0)
	    Ex[i*ny+j] = Ex[i*ny+j] + Cb[i*ny+j]*(Hz[i*ny+j] - (Hzx_psb[(i+1)*npml-1]+Hzy_psb[(i+1)*npml-1]) ) - Cc[i*ny+j]*Fx[i*ny+j];
	  else
	    Ex[i*ny+j] = Ex[i*ny+j] + Cb[i*ny+j]*(Hz[i*ny+j] - Hz[i*ny+(j-1)]) - Cc[i*ny+j]*Fx[i*ny+j];
	}
      for(i=0;i<nx/p;i++)
	for(j=1;j<npml;j++)
	  Ex_psb[i*npml+j] = Ca_psb[i*npml+j]*Ex_psb[i*npml+j] + Cb_psb[i*npml+j]*( ( Hzx_psb[i*npml+j]+Hzy_psb[i*npml+j] )-(Hzx_psb[i*npml+j-1]+Hzy_psb[i*npml+j-1]) );
      if(iteration>=7000)
	for(i=0;i<nx/p;i++)
	  for(j=0;j<ny;j++)
	    Hz_dmy[i*ny+j] += Hz[i*ny+j]*Hz[i*ny+j];
    }


    MPI_Send(Hz,nx*ny/p,MPI_DOUBLE,0,node,MPI_COMM_WORLD);
    //MPI_Send(Hz_dmy,nx*ny/p,MPI_DOUBLE,0,node,MPI_COMM_WORLD);

  }// if(node ==16)


  ///////////////////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////////////////
  ////////////////////////////////     NODE n START    //////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////////////////////////////
  else
  {
    // sigma
    double *sigma_p = (double *)malloc(sizeof(double)*npml);
    // PML 5-1,7-1
    double *Ca_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Cb_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Da_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Db_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Ca_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Cb_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Da_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Db_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));

    // FS
    double *Cb = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Cc = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Db = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Dc = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Fx = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Fy = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Fz = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Gamma_e = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Gamma_h = (double *)malloc(sizeof(double)*(nx/p)*(ny));

    // current source
    double *M = (double *)malloc(sizeof(double)*(nx/p)*(ny));

    // Fields
    double *Ex = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Ey = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Hz = (double *)malloc(sizeof(double)*(nx/p)*(ny));
    double *Hz_dmy = (double *)malloc(sizeof(double)*(nx/p)*(ny));

    double *Ex_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml+1));
    double *Ey_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Hzx_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Hzy_psu = (double *)malloc(sizeof(double)*(nx/p)*(npml));

    double *Ex_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Ey_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Hzx_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));
    double *Hzy_psb = (double *)malloc(sizeof(double)*(nx/p)*(npml));


    // shared components
    // send
    double *Ey_sndu = (double *)malloc(sizeof(double)*npml);
    double *Ey_snd = (double *)malloc(sizeof(double)*ny);
    double *Ey_sndb = (double *)malloc(sizeof(double)*npml);
    double *Hzx_sndu = (double *)malloc(sizeof(double)*npml);
    double *Hzy_sndu = (double *)malloc(sizeof(double)*npml);
    double *Hz_snd = (double *)malloc(sizeof(double)*ny);
    double *Hzx_sndb = (double *)malloc(sizeof(double)*npml);
    double *Hzy_sndb = (double *)malloc(sizeof(double)*npml);

    // receive
    double *Ey_rcvu = (double *)malloc(sizeof(double)*npml);
    double *Ey_rcv = (double *)malloc(sizeof(double)*ny);
    double *Ey_rcvb = (double *)malloc(sizeof(double)*npml);
    double *Hzx_rcvu = (double *)malloc(sizeof(double)*npml);
    double *Hzy_rcvu = (double *)malloc(sizeof(double)*npml);
    double *Hz_rcv = (double *)malloc(sizeof(double)*ny);
    double *Hzx_rcvb = (double *)malloc(sizeof(double)*npml);
    double *Hzy_rcvb = (double *)malloc(sizeof(double)*npml);

    FILE *system;
    char filename[256];


    for(i=0;i<ny;i++)
    {
      if(i<npml)
      {
	Ey_sndu[i] = 0.0;
	Ey_snd[i] = 0.0;
	Ey_sndb[i] = 0.0;
	Hzx_sndu[i] = 0.0;
	Hzy_sndu[i] = 0.0;
	Hz_snd[i] = 0.0;
	Hzx_sndb[i] = 0.0;
	Hzy_sndb[i] = 0.0;
	Ey_rcvu[i] = 0.0;
	Ey_rcv[i] = 0.0;
	Ey_rcvb[i] = 0.0;
	Hzx_rcvu[i] = 0.0;
	Hzy_rcvu[i] = 0.0;
	Hz_rcv[i] = 0.0;
	Hzx_rcvb[i] = 0.0;
	Hzy_rcvb[i] = 0.0;
      }
      else
      {
	Ey_snd[i] = 0.0;
	Hz_snd[i] = 0.0;
	Ey_rcv[i] = 0.0;
	Hz_rcv[i] = 0.0;
      }
    }


    for(i=0;i<npml;i++) 
      sigma_p[i] = (double)((double)sigma_m*(double)((i+1)*(i+1)*(i+1)))/(double)(npml*npml*npml);
    sigma_p[0] = ((sigma_p[0] - 0)/2.0 + sigma_p[0] + (sigma_p[0] + sigma_p[1])/2.0)/3.0;
    for(i=1;i<npml-1;i++) 
      sigma_p[i] = ((sigma_p[i-1]+sigma_p[i])/2.0 + sigma_p[i] + (sigma_p[i] + sigma_p[i+1])/2.0)/3.0;


    for(i=0;i<nx/p;i++)
      for(j=0;j<ny;j++)
      {
	if(j<npml+1)
	{
	  if(j==npml)
	    Ex_psu[i*(npml+1)+j] = 0.0;
	  else
	  {
	    Ex_psu[i*(npml+1)+j] = 0.0;
	    Ey_psu[i*npml+j] = 0.0;
	    Hzx_psu[i*npml+j] = 0.0;
	    Hzy_psu[i*npml+j] = 0.0;

	    Ex_psb[i*npml+j] = 0.0;
	    Ey_psb[i*npml+j] = 0.0;
	    Hzx_psb[i*npml+j] = 0.0;
	    Hzy_psb[i*npml+j] = 0.0;
	  }
	}
	Ex[i*ny+j] = 0.0;
	Ey[i*ny+j] = 0.0;
	Hz[i*ny+j] = 0.0;
	Hz_dmy[i*ny+j] = 0.0;
	M[i*ny+j] = 0.0;
      }

    sprintf(filename,"node%d.dat",node);
    system = fopen(filename,"r");
    if(strcmp(source_mode,"TE")==0)
    {
      for(i=0;i<nx/p;i++) 
	for(j=0;j<npml;j++) 
	{ 
	  Ca_psu[i*npml+j] = (1-sigma_p[j]*dT/(2*EPSILON_0))/(1+sigma_p[j]*dT/(2*EPSILON_0)); 
	  Cb_psu[i*npml+j] = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[j]*dT/(2*EPSILON_0)); 
	  Da_psu[i*npml+j] = (1-SH*sigma_p[j]*dT/(2*MU_0))/(1+SH*sigma_p[j]*dT/(2*MU_0)); 
	  Db_psu[i*npml+j] = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[j]*dT/(2*MU_0)); 

	  Ca_psb[i*npml+j] = (1-sigma_p[npml-1-j]*dT/(2*EPSILON_0))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0)); 
	  Cb_psb[i*npml+j] = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0)); 
	  Da_psb[i*npml+j] = (1-SH*sigma_p[npml-1-j]*dT/(2*MU_0))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0)); 
	  Db_psb[i*npml+j] = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0)); 
	}

      for(j=0;j<ny;j++) 
	for(i=0;i<nx/p;i++) 
	{ 
	  fscanf(system,"%d",&index);
	  Cb[i*ny+j] = sqrt(EPSILON_0/MU_0)*(dT/(materials[index].permitivity*cellsize));
	  Cc[i*ny+j] = dT*materials[index].p_freq_elec*materials[index].p_freq_elec;
	  Db[i*ny+j] = sqrt(MU_0/EPSILON_0)*(dT/(materials[index].permiability*cellsize));
	  Dc[i*ny+j] = dT*materials[index].p_freq_mag*materials[index].p_freq_mag;
	  Fx[i*ny+j] = 0.0;
	  Fy[i*ny+j] = 0.0;
	  Fz[i*ny+j] = 0.0;
	  Gamma_e[i*ny+j] = materials[index].r_rate_elec;
	  Gamma_h[i*ny+j] = materials[index].r_rate_mag;
	}
    }// end of if(strcmp(source_mode,"TE")==0)

    if(strcmp(source_mode,"TM")==0)
    {
      for(i=0;i<nx/p;i++) 
	for(j=0;j<npml;j++) 
	{ 
	  Da_psu[i*npml+j] = (1-sigma_p[j]*dT/(2*EPSILON_0))/(1+sigma_p[j]*dT/(2*EPSILON_0)); 
	  Db_psu[i*npml+j] = -sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[j]*dT/(2*EPSILON_0)); 
	  Ca_psu[i*npml+j] = (1-SH*sigma_p[j]*dT/(2*MU_0))/(1+SH*sigma_p[j]*dT/(2*MU_0)); 
	  Cb_psu[i*npml+j] = -sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[j]*dT/(2*MU_0)); 

	  Da_psb[i*npml+j] = (1-sigma_p[npml-1-j]*dT/(2*EPSILON_0))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0)); 
	  Db_psb[i*npml+j] = -sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*cellsize))/(1+sigma_p[npml-1-j]*dT/(2*EPSILON_0)); 
	  Ca_psb[i*npml+j] = (1-SH*sigma_p[npml-1-j]*dT/(2*MU_0))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0)); 
	  Cb_psb[i*npml+j] = -sqrt(MU_0/EPSILON_0)*(dT/(MU_0*cellsize))/(1+SH*sigma_p[npml-1-j]*dT/(2*MU_0)); 
	}

      for(j=0;j<ny;j++) 
	for(i=0;i<nx/p;i++) 
	{ 
	  fscanf(system,"%d",&index);
	  Db[i*ny+j] = -sqrt(EPSILON_0/MU_0)*(dT/(materials[index].permitivity*cellsize));
	  Dc[i*ny+j] = dT*materials[index].p_freq_elec*materials[index].p_freq_elec;
	  Cb[i*ny+j] = -sqrt(MU_0/EPSILON_0)*(dT/(materials[index].permiability*cellsize));
	  Cc[i*ny+j] = dT*materials[index].p_freq_mag*materials[index].p_freq_mag;
	  Fx[i*ny+j] = 0.0;
	  Fy[i*ny+j] = 0.0;
	  Fz[i*ny+j] = 0.0;
	  Gamma_e[i*ny+j] = materials[index].r_rate_elec;
	  Gamma_h[i*ny+j] = materials[index].r_rate_mag;
	}
    }// end of if(strcmp(source_mode,"TM")==0)
    fclose(system);


    // fields update
    for(iteration=0;iteration<MAXTIMESTEP;iteration++)
    {
      // Current source input
      if(Bp==node)
      {
	for(j=0;j<ny;j++)
	{
	  M[source_xp*ny+j] = exp(-pow(((double)(j-source_yp))/(double)w_of_beam,2))*creal(cexp(-I*(2.0*M_PI*freq*sin(theta)*(double)(j-source_yp)*cellsize/LIGHT_SPEED - 2.0*M_PI*freq*(double)iteration*dT)));
	}
      }
      // data sync.
      for(i=0;i<npml;i++)
      {
	Ey_sndu[i] = Ey_psu[i];
	Ey_sndb[i] = Ey_psb[i];
      }
      for(i=0;i<ny;i++)
	Ey_snd[i] = Ey[i];

      MPI_Irecv(Ey_rcvu,npml,MPI_DOUBLE,node+1,(node+1)*100+1,MPI_COMM_WORLD,&rdsync1);
      MPI_Irecv(Ey_rcv,ny,MPI_DOUBLE,node+1,(node+1)*100+2,MPI_COMM_WORLD,&rdsync2);
      MPI_Irecv(Ey_rcvb,npml,MPI_DOUBLE,node+1,(node+1)*100+3,MPI_COMM_WORLD,&rdsync3);

      MPI_Send(Ey_sndu,npml,MPI_DOUBLE,node-1,node*100+1,MPI_COMM_WORLD);
      MPI_Send(Ey_snd,ny,MPI_DOUBLE,node-1,node*100+2,MPI_COMM_WORLD);
      MPI_Send(Ey_sndb,npml,MPI_DOUBLE,node-1,node*100+3,MPI_COMM_WORLD);

      MPI_Wait(&rdsync1, &sdsync1);
      MPI_Wait(&rdsync2, &sdsync2);
      MPI_Wait(&rdsync3, &sdsync3);


      // Dispersion function
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	  Fz[i*ny+j] = dT*Hz[i*ny+j] + exp(-Gamma_h[i*ny+j]*dT)*Fz[i*ny+j];

      // Hz
      for(i=0;i<nx/p;i++)
	for(j=0;j<npml-1;j++)
	{
	  if(i==nx/p-1)
	  {
	    Hzx_psu[i*npml+j] = Da_psu[i*npml+j]*Hzx_psu[i*npml+j] - Db_psu[i*npml+j]*(Ey_rcvu[j] - Ey_psu[i*npml+j]);
	    Hzy_psu[i*npml+j] = Da_psu[i*npml+j]*Hzy_psu[i*npml+j] + Db_psu[i*npml+j]*(Ex_psu[i*(npml+1)+(j+1)] - Ex_psu[i*(npml+1)+j]);
	  }
	  else
	  {
	    Hzx_psu[i*npml+j] = Da_psu[i*npml+j]*Hzx_psu[i*npml+j] - Db_psu[i*npml+j]*(Ey_psu[(i+1)*npml+j] - Ey_psu[i*npml+j]);
	    Hzy_psu[i*npml+j] = Da_psu[i*npml+j]*Hzy_psu[i*npml+j] + Db_psu[i*npml+j]*(Ex_psu[i*(npml+1)+(j+1)] - Ex_psu[i*(npml+1)+j]);
	  }
	}
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	{
	  if(i==nx/p-1 && j==ny-1)
	    Hz[i*ny+j] = Hz[i*ny+j] + Db[i*ny+j]*(Ex_psu[i*(npml+1)] - Ex[i*ny+j] - Ey_rcv[j] + Ey[i*ny+j] - cellsize*M[i*ny+j]) - Dc[i*ny+j]*Fz[i*ny+j];
	  else if(i==nx/p-1)
	    Hz[i*ny+j] = Hz[i*ny+j] + Db[i*ny+j]*(Ex[i*ny+(j+1)] - Ex[i*ny+j] - Ey_rcv[j] + Ey[i*ny+j] - cellsize*M[i*ny+j]) - Dc[i*ny+j]*Fz[i*ny+j];
	  else if(j==ny-1)
	    Hz[i*ny+j] = Hz[i*ny+j] + Db[i*ny+j]*(Ex_psu[i*(npml+1)] - Ex[i*ny+j] - Ey[(i+1)*ny+j] + Ey[i*ny+j] - cellsize*M[i*ny+j]) - Dc[i*ny+j]*Fz[i*ny+j];
	  else
	    Hz[i*ny+j] = Hz[i*ny+j] + Db[i*ny+j]*(Ex[i*ny+(j+1)] - Ex[i*ny+j] - Ey[(i+1)*ny+j] + Ey[i*ny+j] - cellsize*M[i*ny+j]) - Dc[i*ny+j]*Fz[i*ny+j];
	}
      for(i=0;i<nx/p;i++)
	for(j=1;j<npml;j++)
	{
	  if(i==nx/p-1 && j==npml-1)
	  {
	    Hzx_psb[i*npml+j] = Da_psb[i*npml+j]*Hzx_psb[i*npml+j] - Db_psb[i*npml+j]*(Ey_rcvb[j] - Ey_psb[i*npml+j]);
	    Hzy_psb[i*npml+j] = Da_psb[i*npml+j]*Hzy_psb[i*npml+j] + Db_psb[i*npml+j]*(Ex[i*ny] - Ex_psb[i*npml+j]);
	  }
	  else if(i==nx/p-1)
	  {
	    Hzx_psb[i*npml+j] = Da_psb[i*npml+j]*Hzx_psb[i*npml+j] - Db_psb[i*npml+j]*(Ey_rcvb[j] - Ey_psb[i*npml+j]);
	    Hzy_psb[i*npml+j] = Da_psb[i*npml+j]*Hzy_psb[i*npml+j] + Db_psb[i*npml+j]*(Ex_psb[i*npml+j+1] - Ex_psb[i*npml+j]);
	  }
	  else if(j==npml-1)
	  {
	    Hzx_psb[i*npml+j] = Da_psb[i*npml+j]*Hzx_psb[i*npml+j] - Db_psb[i*npml+j]*(Ey_psb[(i+1)*npml+j] - Ey_psb[i*npml+j]);
	    Hzy_psb[i*npml+j] = Da_psb[i*npml+j]*Hzy_psb[i*npml+j] + Db_psb[i*npml+j]*(Ex[i*ny] - Ex_psb[i*npml+j]);
	  }
	  else
	  {
	    Hzx_psb[i*npml+j] = Da_psb[i*npml+j]*Hzx_psb[i*npml+j] - Db_psb[i*npml+j]*(Ey_psb[(i+1)*npml+j] - Ey_psb[i*npml+j]);
	    Hzy_psb[i*npml+j] = Da_psb[i*npml+j]*Hzy_psb[i*npml+j] + Db_psb[i*npml+j]*(Ex_psb[i*npml+j+1] - Ex_psb[i*npml+j]);
	  }
	}


      // data sync.
      for(i=0;i<npml;i++)
      {
	Hzx_sndu[i] = Hzx_psu[(nx/p-1)*npml+i];
	Hzy_sndu[i] = Hzy_psu[(nx/p-1)*npml+i];
	Hzx_sndb[i] = Hzx_psb[(nx/p-1)*npml+i];
	Hzy_sndb[i] = Hzy_psb[(nx/p-1)*npml+i];
      }
      for(i=0;i<ny;i++)
	Hz_snd[i] = Hz[(nx/p-1)*ny+i];

      MPI_Irecv(Hzx_rcvu,npml,MPI_DOUBLE,node-1,(node-1)*100+4,MPI_COMM_WORLD,&rdsync4);
      MPI_Irecv(Hzy_rcvu,npml,MPI_DOUBLE,node-1,(node-1)*100+5,MPI_COMM_WORLD,&rdsync5);
      MPI_Irecv(Hz_rcv,ny,MPI_DOUBLE,node-1,(node-1)*100+6,MPI_COMM_WORLD,&rdsync6);
      MPI_Irecv(Hzx_rcvb,npml,MPI_DOUBLE,node-1,(node-1)*100+7,MPI_COMM_WORLD,&rdsync7);
      MPI_Irecv(Hzy_rcvb,npml,MPI_DOUBLE,node-1,(node-1)*100+8,MPI_COMM_WORLD,&rdsync8);

      MPI_Send(Hzx_sndu,npml,MPI_DOUBLE,node+1,node*100+4,MPI_COMM_WORLD);
      MPI_Send(Hzy_sndu,npml,MPI_DOUBLE,node+1,node*100+5,MPI_COMM_WORLD);
      MPI_Send(Hz_snd,ny,MPI_DOUBLE,node+1,node*100+6,MPI_COMM_WORLD);
      MPI_Send(Hzx_sndb,npml,MPI_DOUBLE,node+1,node*100+7,MPI_COMM_WORLD);
      MPI_Send(Hzy_sndb,npml,MPI_DOUBLE,node+1,node*100+8,MPI_COMM_WORLD);

      MPI_Wait(&rdsync4, &sdsync4);
      MPI_Wait(&rdsync5, &sdsync5);
      MPI_Wait(&rdsync6, &sdsync6);
      MPI_Wait(&rdsync7, &sdsync7);
      MPI_Wait(&rdsync8, &sdsync8);


      // Dispersion function
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	  Fy[i*ny+j] = dT*Ey[i*ny+j] + exp(-Gamma_e[i*ny+j]*dT)*Fy[i*ny+j];

      // Ey
      for(i=0;i<nx/p;i++)
	for(j=0;j<npml-1;j++)
	{
	  if(i==0)
	    Ey_psu[i*npml+j] = Ca_psu[i*npml+j]*Ey_psu[i*npml+j] - Cb_psu[i*npml+j]*( (Hzx_psu[i*npml+j]+Hzy_psu[i*npml+j] ) - ( Hzx_rcvu[j]+Hzy_rcvu[j]) );
	  else
	    Ey_psu[i*npml+j] = Ca_psu[i*npml+j]*Ey_psu[i*npml+j] - Cb_psu[i*npml+j]*( (Hzx_psu[i*npml+j]+Hzy_psu[i*npml+j] ) - ( Hzx_psu[(i-1)*npml+j]+Hzy_psu[(i-1)*npml+j]) );
	}
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	{
	  if(i==0)
	    Ey[i*ny+j] = Ey[i*ny+j] - Cb[i*ny+j]*(Hz[i*ny+j] - Hz_rcv[j]) - Cc[i*ny+j]*Fy[i*ny+j];
	  else
	    Ey[i*ny+j] = Ey[i*ny+j] - Cb[i*ny+j]*(Hz[i*ny+j] - Hz[(i-1)*ny+j]) - Cc[i*ny+j]*Fy[i*ny+j];
	}
      for(i=0;i<nx/p;i++)
	for(j=1;j<npml-1;j++)
	{
	  if(i==0)
	    Ey_psb[i*npml+j] = Ca_psb[i*npml+j]*Ey_psb[i*npml+j] - Cb_psb[i*npml+j]*( (Hzx_psb[i*npml+j]+Hzy_psb[i*npml+j] ) - (Hzx_rcvb[j]+Hzy_rcvb[j] ) );
	  else
	    Ey_psb[i*npml+j] = Ca_psb[i*npml+j]*Ey_psb[i*npml+j] - Cb_psb[i*npml+j]*( (Hzx_psb[i*npml+j]+Hzy_psb[i*npml+j] ) - (Hzx_psb[(i-1)*npml+j]+Hzy_psb[(i-1)*npml+j] ) );
	}

      // Dispersion function
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	  Fx[i*ny+j] = dT*Ex[i*ny+j] + exp(-Gamma_e[i*ny+j]*dT)*Fx[i*ny+j];


      // Ex
      for(i=0;i<nx/p;i++)
	for(j=0;j<npml;j++)
	{
	  if(j==0)
	    Ex_psu[i*(npml+1)+j] = Ca_psu[i*npml+j]*Ex_psu[i*(npml+1)+j] + Cb_psu[i*npml+j]*( ( Hzx_psu[i*npml+j]+Hzy_psu[i*npml+j] ) - ( Hz[(i+1)*ny-1] ) );
	  else
	    Ex_psu[i*(npml+1)+j] = Ca_psu[i*npml+j]*Ex_psu[i*(npml+1)+j] + Cb_psu[i*npml+j]*( ( Hzx_psu[i*npml+j]+Hzy_psu[i*npml+j] ) - ( Hzx_psu[i*npml+j-1]+Hzy_psu[i*npml+j-1] ) );
	}
      for(i=0;i<nx/p;i++)
	for(j=0;j<ny;j++)
	{
	  if(j==0)
	    Ex[i*ny+j] = Ex[i*ny+j] + Cb[i*ny+j]*(Hz[i*ny+j] - (Hzx_psb[(i+1)*npml-1]+Hzy_psb[(i+1)*npml-1]) ) - Cc[i*ny+j]*Fx[i*ny+j];
	  else
	    Ex[i*ny+j] = Ex[i*ny+j] + Cb[i*ny+j]*(Hz[i*ny+j] - Hz[i*ny+(j-1)]) - Cc[i*ny+j]*Fx[i*ny+j];
	}
      for(i=0;i<nx/p;i++)
	for(j=1;j<npml;j++)
	  Ex_psb[i*npml+j] = Ca_psb[i*npml+j]*Ex_psb[i*npml+j] + Cb_psb[i*npml+j]*( ( Hzx_psb[i*npml+j]+Hzy_psb[i*npml+j] )-(Hzx_psb[i*npml+j-1]+Hzy_psb[i*npml+j-1]) );
      if(iteration>=7000)
	for(i=0;i<nx/p;i++)
	  for(j=0;j<ny;j++)
	    Hz_dmy[i*ny+j] += Hz[i*ny+j]*Hz[i*ny+j];



    }
    MPI_Send(Hz,nx*ny/p,MPI_DOUBLE,0,node,MPI_COMM_WORLD);
    //MPI_Send(Hz_dmy,nx*ny/p,MPI_DOUBLE,0,node,MPI_COMM_WORLD);


  }// if(node ==1)

  MPI_Finalize();
  return 0;
}
